@@ -4,7 +4,7 @@
 # conditional.py, provides a class to store field comparison functions
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -13,7 +13,11 @@
 #******************************************************************************
 
 import re
-from PyQt4 import QtCore, QtGui
+import enum
+from PyQt5.QtCore import QSize, Qt, pyqtSignal
+from PyQt5.QtWidgets import (QComboBox, QDialog, QGroupBox, QHBoxLayout,
+                             QLabel, QLineEdit, QListWidget, QPushButton,
+                             QSizePolicy, QVBoxLayout)
 import treeformats
 import configdialog
 import undo
@@ -53,7 +57,8 @@ class Conditional:
         self.nodeFormatNames = set()
         if nodeFormatName:
             self.nodeFormatNames.add(nodeFormatName)
-            nodeFormats = globalref.mainControl.activeControl.model.formats
+            nodeFormats = (globalref.mainControl.activeControl.structure.
+                           treeFormats)
             for nodeType in nodeFormats[nodeFormatName].derivedTypes:
                 self.nodeFormatNames.add(nodeType.name)
 
@@ -64,7 +69,7 @@ class Conditional:
             node -- the node to check for a field match
         """
         if (self.nodeFormatNames and
-            node.formatName not in self.nodeFormatNames):
+            node.formatRef.name not in self.nodeFormatNames):
             return False
         result = True
         for conditon in self.conditionLines:
@@ -129,7 +134,7 @@ class ConditionLine:
             prevResult -- the result to combine with the boolOper
         """
         try:
-            field = node.nodeFormat().fieldDict[self.fieldName]
+            field = node.formatRef.fieldDict[self.fieldName]
         except KeyError:
             if self.boolOper == 'and':
                 return False
@@ -191,14 +196,16 @@ class StringOps(str):
         return False
 
 
-class ConditionDialog(QtGui.QDialog):
+FindDialogType = enum.Enum('FindDialogType',
+                           'typeDialog findDialog filterDialog')
+
+class ConditionDialog(QDialog):
     """Dialog for defining field condition tests.
 
     Used for defining conditional types (modal), for finding by condition
     (nonmodal) and for filtering by condition (nonmodal).
     """
-    dialogShown = QtCore.pyqtSignal(bool)
-    typeDialog, findDialog, filterDialog = range(3)
+    dialogShown = pyqtSignal(bool)
     def __init__(self, dialogType, caption, nodeFormat=None, parent=None):
         """Create the conditional dialog.
 
@@ -219,77 +226,99 @@ class ConditionDialog(QtGui.QDialog):
         self.fieldNames = []
         if nodeFormat:
             self.fieldNames = nodeFormat.fieldNames()
-        topLayout = QtGui.QVBoxLayout(self)
+        topLayout = QVBoxLayout(self)
 
-        if dialogType == ConditionDialog.typeDialog:
-            self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint |
-                                QtCore.Qt.WindowSystemMenuHint)
+        if dialogType == FindDialogType.typeDialog:
+            self.setWindowFlags(Qt.Dialog | Qt.WindowTitleHint |
+                                Qt.WindowCloseButtonHint)
         else:
-            self.setAttribute(QtCore.Qt.WA_QuitOnClose, False)
-            self.setWindowFlags(QtCore.Qt.Window |
-                                QtCore.Qt.WindowStaysOnTopHint)
-            typeBox = QtGui.QGroupBox(_('Node Type'))
+            self.setAttribute(Qt.WA_QuitOnClose, False)
+            self.setWindowFlags(Qt.Window |
+                                Qt.WindowStaysOnTopHint)
+            typeBox = QGroupBox(_('Node Type'))
             topLayout.addWidget(typeBox)
-            typeLayout = QtGui.QVBoxLayout(typeBox)
-            self.typeCombo = QtGui.QComboBox()
+            typeLayout = QVBoxLayout(typeBox)
+            self.typeCombo = QComboBox()
             typeLayout.addWidget(self.typeCombo)
             self.typeCombo.currentIndexChanged.connect(self.updateDataType)
 
-        self.mainLayout = QtGui.QVBoxLayout()
+        self.mainLayout = QVBoxLayout()
         topLayout.addLayout(self.mainLayout)
 
-        upCtrlLayout = QtGui.QHBoxLayout()
+        upCtrlLayout = QHBoxLayout()
         topLayout.addLayout(upCtrlLayout)
-        upCtrlLayout.addStretch()
-        addButton = QtGui.QPushButton(_('&Add New Rule'))
+        addButton = QPushButton(_('&Add New Rule'))
         upCtrlLayout.addWidget(addButton)
         addButton.clicked.connect(self.addNewRule)
-        self.removeButton = QtGui.QPushButton(_('&Remove Rule'))
+        self.removeButton = QPushButton(_('&Remove Rule'))
         upCtrlLayout.addWidget(self.removeButton)
         self.removeButton.clicked.connect(self.removeRule)
+        upCtrlLayout.addStretch()
 
-        if dialogType == ConditionDialog.typeDialog:
-            okButton = QtGui.QPushButton(_('&OK'))
+        if dialogType == FindDialogType.typeDialog:
+            okButton = QPushButton(_('&OK'))
             upCtrlLayout.addWidget(okButton)
             okButton.clicked.connect(self.accept)
-            cancelButton = QtGui.QPushButton(_('&Cancel'))
+            cancelButton = QPushButton(_('&Cancel'))
             upCtrlLayout.addWidget(cancelButton)
             cancelButton.clicked.connect(self.reject)
         else:
             self.removeButton.setEnabled(False)
-            self.retrieveButton = QtGui.QPushButton(_('R&etrieve Rules...'))
-            upCtrlLayout.addWidget(self.retrieveButton)
-            self.retrieveButton.clicked.connect(self.retrieveRules)
-            saveButton = QtGui.QPushButton(_('&Save Rules...'))
-            upCtrlLayout.addWidget(saveButton)
-            saveButton.clicked.connect(self.saveRules)
+            saveBox = QGroupBox(_('Saved Rules'))
+            topLayout.addWidget(saveBox)
+            saveLayout = QVBoxLayout(saveBox)
+            self.saveListBox = SmallListWidget()
+            saveLayout.addWidget(self.saveListBox)
+            self.saveListBox.itemDoubleClicked.connect(self.loadSavedRule)
+            nameLayout = QHBoxLayout()
+            saveLayout.addLayout(nameLayout)
+            label = QLabel(_('Name:'))
+            nameLayout.addWidget(label)
+            self.saveNameEdit = QLineEdit()
+            nameLayout.addWidget(self.saveNameEdit)
+            self.saveNameEdit.textChanged.connect(self.updateSaveEnable)
+            saveButtonLayout = QHBoxLayout()
+            saveLayout.addLayout(saveButtonLayout)
+            self.loadSavedButton = QPushButton(_('&Load'))
+            saveButtonLayout.addWidget(self.loadSavedButton)
+            self.loadSavedButton.clicked.connect(self.loadSavedRule)
+            self.saveButton = QPushButton(_('&Save'))
+            saveButtonLayout.addWidget(self.saveButton)
+            self.saveButton.clicked.connect(self.saveRule)
+            self.saveButton.setEnabled(False)
+            self.delSavedButton = QPushButton(_('&Delete'))
+            saveButtonLayout.addWidget(self.delSavedButton)
+            self.delSavedButton.clicked.connect(self.deleteRule)
+            saveButtonLayout.addStretch()
 
-            lowCtrlLayout = QtGui.QHBoxLayout()
+            if dialogType == FindDialogType.findDialog:
+                self.resultLabel = QLabel()
+                topLayout.addWidget(self.resultLabel)
+            lowCtrlLayout = QHBoxLayout()
             topLayout.addLayout(lowCtrlLayout)
-            lowCtrlLayout.addStretch()
-            if dialogType == ConditionDialog.findDialog:
-                previousButton = QtGui.QPushButton(_('Find &Previous'))
+            if dialogType == FindDialogType.findDialog:
+                previousButton = QPushButton(_('Find &Previous'))
                 lowCtrlLayout.addWidget(previousButton)
                 previousButton.clicked.connect(self.findPrevious)
-                nextButton = QtGui.QPushButton(_('Find &Next'))
+                nextButton = QPushButton(_('Find &Next'))
                 nextButton.setDefault(True)
                 lowCtrlLayout.addWidget(nextButton)
                 nextButton.clicked.connect(self.findNext)
-                self.resultLabel = QtGui.QLabel()
-                topLayout.addWidget(self.resultLabel)
             else:
-                filterButton = QtGui.QPushButton(_('&Filter'))
+                filterButton = QPushButton(_('&Filter'))
                 lowCtrlLayout.addWidget(filterButton)
                 filterButton.clicked.connect(self.startFilter)
-                self.endFilterButton = QtGui.QPushButton(_('&End Filter'))
+                self.endFilterButton = QPushButton(_('&End Filter'))
                 lowCtrlLayout.addWidget(self.endFilterButton)
                 self.endFilterButton.setEnabled(False)
                 self.endFilterButton.clicked.connect(self.endFilter)
-            closeButton = QtGui.QPushButton(_('&Close'))
+            lowCtrlLayout.addStretch()
+            closeButton = QPushButton(_('&Close'))
             lowCtrlLayout.addWidget(closeButton)
             closeButton.clicked.connect(self.close)
             origTypeName = nodeFormat.name if nodeFormat else ''
             self.loadTypeNames(origTypeName)
+            self.loadSavedNames()
         self.ruleList.append(ConditionRule(1, self.fieldNames))
         self.mainLayout.addWidget(self.ruleList[0])
 
@@ -301,14 +330,14 @@ class ConditionDialog(QtGui.QDialog):
             combineBool -- the boolean op for combining with the previous rule
         """
         if self.ruleList:
-            boolBox = QtGui.QComboBox()
+            boolBox = QComboBox()
             boolBox.setEditable(False)
             self.combiningBoxes.append(boolBox)
             boolBox.addItems([_(op) for op in _boolOper])
             if combineBool != 'and':
                 boolBox.setCurrentIndex(1)
             self.mainLayout.insertWidget(len(self.ruleList) * 2 - 1, boolBox,
-                                        0, QtCore.Qt.AlignHCenter)
+                                        0, Qt.AlignHCenter)
         rule = ConditionRule(len(self.ruleList) + 1, self.fieldNames)
         self.ruleList.append(rule)
         self.mainLayout.insertWidget(len(self.ruleList) * 2 - 2, rule)
@@ -323,7 +352,7 @@ class ConditionDialog(QtGui.QDialog):
                 del self.combiningBoxes[-1]
             self.ruleList[-1].hide()
             del self.ruleList[-1]
-            if self.dialogType == ConditionDialog.typeDialog:
+            if self.dialogType == FindDialogType.typeDialog:
                 self.removeButton.setEnabled(len(self.ruleList) > 0)
             else:
                 self.removeButton.setEnabled(len(self.ruleList) > 1)
@@ -385,7 +414,7 @@ class ConditionDialog(QtGui.QDialog):
         """
         if not origTypeName:
             origTypeName = self.typeCombo.currentText()
-        nodeFormats = globalref.mainControl.activeControl.model.formats
+        nodeFormats = globalref.mainControl.activeControl.structure.treeFormats
         self.typeCombo.blockSignals(True)
         self.typeCombo.clear()
         self.typeCombo.addItem(_allTypeEntry)
@@ -400,7 +429,6 @@ class ConditionDialog(QtGui.QDialog):
                     self.endFilter()
                 self.clearRules()
         self.typeCombo.blockSignals(False)
-        self.retrieveButton.setEnabled(len(nodeFormats.savedConditions()) > 0)
         self.updateDataType()
 
     def updateDataType(self):
@@ -409,7 +437,7 @@ class ConditionDialog(QtGui.QDialog):
         typeName = self.typeCombo.currentText()
         if not typeName:
             return
-        nodeFormats = globalref.mainControl.activeControl.model.formats
+        nodeFormats = globalref.mainControl.activeControl.structure.treeFormats
         if typeName == _allTypeEntry:
             fieldNameSet = set()
             for typeFormat in nodeFormats.values():
@@ -426,11 +454,43 @@ class ConditionDialog(QtGui.QDialog):
                 break
             rule.reloadFieldBox(self.fieldNames, currentField)
 
+    def loadSavedNames(self, updateOtherDialog=False):
+        """Refresh the list of saved rule names.
+        """
+        selNum = 0
+        if self.saveListBox.count():
+            selNum = self.saveListBox.currentRow()
+        self.saveListBox.clear()
+        nodeFormats = globalref.mainControl.activeControl.structure.treeFormats
+        savedRules = nodeFormats.savedConditions()
+        ruleNames = sorted(list(savedRules.keys()))
+        if ruleNames:
+            self.saveListBox.addItems(ruleNames)
+            if selNum >= len(ruleNames):
+                selNum = len(ruleNames) - 1
+            self.saveListBox.setCurrentRow(selNum)
+        self.loadSavedButton.setEnabled(len(ruleNames) > 0)
+        self.delSavedButton.setEnabled(len(ruleNames) > 0)
+        if updateOtherDialog:
+            if (self != globalref.mainControl.findConditionDialog and
+                globalref.mainControl.findConditionDialog and
+                globalref.mainControl.findConditionDialog.isVisible()):
+                globalref.mainControl.findConditionDialog.loadSavedNames()
+            elif (self != globalref.mainControl.filterConditionDialog and
+                  globalref.mainControl.filterConditionDialog and
+                  globalref.mainControl.filterConditionDialog .isVisible()):
+                globalref.mainControl.filterConditionDialog.loadSavedNames()
+
+    def updateSaveEnable(self):
+        """Set the save rule button enabled based on save name entry.
+        """
+        self.saveButton.setEnabled(len(self.saveNameEdit.text()))
+
     def updateFilterControls(self):
         """Set filter button status based on active window changes.
         """
         window = globalref.mainControl.activeControl.activeWindow
-        if window.isFiltering():
+        if window.treeFilterView:
             filterView = window.treeFilterView
             conditional = filterView.conditionalFilter
             self.setCondition(conditional, conditional.origNodeFormatName)
@@ -438,54 +498,53 @@ class ConditionDialog(QtGui.QDialog):
         else:
             self.endFilterButton.setEnabled(False)
 
-    def retrieveRules(self):
-        """Show a menu to retrieve stored rules.
+    def loadSavedRule(self):
+        """Load the current saved rule into the dialog.
         """
-        modelRef = globalref.mainControl.activeControl.model
-        nodeFormats = modelRef.formats
+        nodeFormats = globalref.mainControl.activeControl.structure.treeFormats
         savedRules = nodeFormats.savedConditions()
-        ruleNames = sorted(list(savedRules.keys()))
-        dlg = RuleRetrieveDialog(ruleNames, self)
-        if dlg.exec_() == QtGui.QDialog.Accepted:
-            if dlg.selectedRule:
-                conditional = savedRules[dlg.selectedRule]
-                self.setCondition(conditional, conditional.origNodeFormatName)
-            if dlg.removedRules:
-                undo.FormatUndo(modelRef.undoList, nodeFormats,
-                                treeformats.TreeFormats())
-                for ruleName in dlg.removedRules:
-                    conditional = savedRules[ruleName]
-                    if conditional.origNodeFormatName:
-                        typeFormat = nodeFormats[conditional.
-                                                 origNodeFormatName]
-                        del typeFormat.savedConditionText[ruleName]
-                    else:
-                        del nodeFormats.savedConditionText[ruleName]
-                self.retrieveButton.setEnabled(len(nodeFormats.
-                                                   savedConditions()) > 0)
-                globalref.mainControl.activeControl.setModified()
+        ruleName = self.saveListBox.currentItem().text()
+        conditional = savedRules[ruleName]
+        self.setCondition(conditional, conditional.origNodeFormatName)
 
-    def saveRules(self):
-        """Prompt for a name for storing these rules.
+    def saveRule(self):
+        """Save the current rule settings.
         """
-        modelRef = globalref.mainControl.activeControl.model
-        nodeFormats = modelRef.formats
-        usedNames = set(nodeFormats.savedConditions().keys())
-        dlg = configdialog.NameEntryDialog(_('Save Rules'),
-                                           _('Enter a descriptive name'), '',
-                                           '', usedNames, self)
-        if dlg.exec_() == QtGui.QDialog.Accepted:
-            undo.FormatUndo(modelRef.undoList, nodeFormats,
-                            treeformats.TreeFormats())
-            typeName = self.typeCombo.currentText()
-            if typeName == _allTypeEntry:
-                nodeFormat = nodeFormats
-            else:
-                nodeFormat = nodeFormats[typeName]
-            nodeFormat.savedConditionText[dlg.text] = (self.conditional().
-                                                       conditionStr())
-            self.retrieveButton.setEnabled(True)
-            globalref.mainControl.activeControl.setModified()
+        name = self.saveNameEdit.text()
+        self.saveNameEdit.setText('')
+        treeStructure = globalref.mainControl.activeControl.structure
+        undo.FormatUndo(treeStructure.undoList, treeStructure.treeFormats,
+                        treeformats.TreeFormats())
+        typeName = self.typeCombo.currentText()
+        if typeName == _allTypeEntry:
+            nodeFormat = treeStructure.treeFormats
+        else:
+            nodeFormat = treeStructure.treeFormats[typeName]
+        nodeFormat.savedConditionText[name] = (self.conditional().
+                                               conditionStr())
+        self.loadSavedNames(True)
+        self.saveListBox.setCurrentItem(self.saveListBox.
+                                        findItems(name, Qt.MatchExactly)[0])
+        globalref.mainControl.activeControl.setModified()
+
+    def deleteRule(self):
+        """Remove the current saved rule.
+        """
+        treeStructure = globalref.mainControl.activeControl.structure
+        nodeFormats = treeStructure.treeFormats
+        undo.FormatUndo(treeStructure.undoList, nodeFormats,
+                        treeformats.TreeFormats())
+        savedRules = nodeFormats.savedConditions()
+        ruleName = self.saveListBox.currentItem().text()
+        conditional = savedRules[ruleName]
+        if conditional.origNodeFormatName:
+            typeFormat = nodeFormats[conditional.
+                                     origNodeFormatName]
+            del typeFormat.savedConditionText[ruleName]
+        else:
+            del nodeFormats.savedConditionText[ruleName]
+        self.loadSavedNames(True)
+        globalref.mainControl.activeControl.setModified()
 
     def find(self, forward=True):
         """Find another match in the indicated direction.
@@ -513,19 +572,17 @@ class ConditionDialog(QtGui.QDialog):
         """Start filtering nodes.
         """
         window = globalref.mainControl.activeControl.activeWindow
-        filterView = window.treeFilterView
+        filterView = window.filterView()
         filterView.conditionalFilter = self.conditional()
         filterView.updateContents()
-        window.treeStack.setCurrentWidget(filterView)
         self.endFilterButton.setEnabled(True)
 
     def endFilter(self):
         """Stop filtering nodes.
         """
         window = globalref.mainControl.activeControl.activeWindow
-        window.treeStack.setCurrentWidget(window.treeView)
+        window.removeFilterView()
         self.endFilterButton.setEnabled(False)
-        globalref.mainControl.currentStatusBar().clearMessage()
 
     def closeEvent(self, event):
         """Signal that the dialog is closing.
@@ -536,7 +593,7 @@ class ConditionDialog(QtGui.QDialog):
         self.dialogShown.emit(False)
 
 
-class ConditionRule(QtGui.QGroupBox):
+class ConditionRule(QGroupBox):
     """Group boxes for conditional rules in the ConditionDialog.
     """
     def __init__(self, num, fieldNames, parent=None):
@@ -550,19 +607,19 @@ class ConditionRule(QtGui.QGroupBox):
         super().__init__(parent)
         self.fieldNames = fieldNames
         self.setTitle(_('Rule {0}').format(num))
-        layout = QtGui.QHBoxLayout(self)
-        self.fieldBox = QtGui.QComboBox()
+        layout = QHBoxLayout(self)
+        self.fieldBox = QComboBox()
         self.fieldBox.setEditable(False)
         self.fieldBox.addItems(fieldNames)
         layout.addWidget(self.fieldBox)
 
-        self.operBox = QtGui.QComboBox()
+        self.operBox = QComboBox()
         self.operBox.setEditable(False)
         self.operBox.addItems([_(op) for op in _operators])
         layout.addWidget(self.operBox)
         self.operBox.currentIndexChanged.connect(self.changeOper)
 
-        self.editor = QtGui.QLineEdit()
+        self.editor = QLineEdit()
         layout.addWidget(self.editor)
         self.fieldBox.setFocus()
 
@@ -613,58 +670,25 @@ class ConditionRule(QtGui.QGroupBox):
         self.fieldBox.setEnabled(realOp)
 
 
-class RuleRetrieveDialog(QtGui.QDialog):
-    """Dialog to select saved conditional rules for retrieval or removal.
+class SmallListWidget(QListWidget):
+    """ListWidget with a smaller size hint.
     """
-    def __init__(self, ruleNames, parent=None):
-        """Initialize the rule retrieval dialog.
+    def __init__(self, parent=None):
+        """Initialize the widget.
 
         Arguments:
-            ruleNames -- a list of rulenames to show
-            parent -- the parent overall dialog
+            parent -- the parent, if given
         """
         super().__init__(parent)
-        self.ruleNames = ruleNames
-        self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint |
-                            QtCore.Qt.WindowSystemMenuHint)
-        self.setWindowTitle(_('Retrieve Rules'))
-        self.selectedRule = ''
-        self.removedRules = []
 
-        topLayout = QtGui.QVBoxLayout(self)
-        label = QtGui.QLabel(_('Select rule set to retrieve:'))
-        topLayout.addWidget(label)
-        self.listBox = QtGui.QListWidget()
-        topLayout.addWidget(self.listBox)
-        self.listBox.addItems(ruleNames)
-        self.listBox.setCurrentRow (0)
-        self.listBox.itemDoubleClicked.connect(self.accept)
-
-        ctrlLayout = QtGui.QHBoxLayout()
-        topLayout.addLayout(ctrlLayout)
-        removeButton = QtGui.QPushButton(_('Remove Rule'))
-        ctrlLayout.addWidget(removeButton)
-        removeButton.clicked.connect(self.removeRule)
-        ctrlLayout.addStretch()
-        okButton = QtGui.QPushButton(_('&OK'))
-        ctrlLayout.addWidget(okButton)
-        okButton.clicked.connect(self.accept)
-        cancelButton = QtGui.QPushButton(_('&Cancel'))
-        ctrlLayout.addWidget(cancelButton)
-        cancelButton.clicked.connect(self.reject)
-
-    def removeRule(self):
-        """Remove the currently selected rule.
+    def sizeHint(self):
+        """Return smaller height.
         """
-        currentItem = self.listBox.currentItem()
-        if currentItem:
-            self.removedRules.append(currentItem.text())
-            self.listBox.takeItem(self.listBox.currentRow())
-
-    def accept(self):
-        """Recored results before closing.
-        """
-        currentItem = self.listBox.currentItem()
-        if currentItem:
-            self.selectedRule = currentItem.text()
-        return super().accept()
+        if self.count():
+            rowHeight = self.sizeHintForRow(0)
+        else:
+            self.addItem('tmp')
+            rowHeight = self.sizeHintForRow(0)
+            self.takeItem(0)
+        newHeight = rowHeight * 3 + self.frameWidth() * 2
+        return QSize(super().sizeHint().width(), newHeight)
