@@ -4,7 +4,7 @@
 # dataeditview.py, provides a class for the data edit right-hand view
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2018, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -12,8 +12,13 @@
 # but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
-import os.path
-from PyQt4 import QtCore, QtGui
+from PyQt5.QtCore import QEvent, QPointF, QRectF, QSize, Qt, pyqtSignal
+from PyQt5.QtGui import (QKeySequence, QPainterPath, QPalette, QPen,
+                         QSyntaxHighlighter, QTextCharFormat, QTextCursor,
+                         QTextDocument)
+from PyQt5.QtWidgets import (QAbstractItemView, QApplication,
+                             QStyledItemDelegate, QTableWidget,
+                             QTableWidgetItem)
 import treenode
 import undo
 import urltools
@@ -23,29 +28,28 @@ import globalref
 _minColumnWidth = 80
 defaultFont = None
 
-class DataEditCell(QtGui.QTableWidgetItem):
+class DataEditCell(QTableWidgetItem):
     """Class override for data edit view cells.
     Used for the cells with editable content.
     """
-    def __init__(self, node, field, titleCellRef, typeCellRef, idCellRef=None):
+    def __init__(self, spot, field, titleCellRef, typeCellRef):
         """Initialize the editable cells in the data edit view.
 
         Arguments:
-            node -- the node referenced by this cell
+            spot -- the spot referenced by this cell
             field -- the field object referenced by this cell
             titleCellRef -- the title cell to update based on data changes
             typeCellRef -- the format type cell to update based on type changes
-            idCellRef -- the unique ID cell to update based on data changes
         """
         super().__init__()
-        self.node = node
+        self.spot = spot
+        self.node = spot.nodeRef
         self.field = field
         self.titleCellRef = titleCellRef
         self.typeCellRef = typeCellRef
-        self.idCellRef = idCellRef
         self.errorFlag = False
         # store doc to speed up delegate sizeHint and paint calls
-        self.doc = QtGui.QTextDocument()
+        self.doc = QTextDocument()
         self.doc.setDefaultFont(defaultFont)
         self.doc.setDocumentMargin(6)
         self.updateText()
@@ -62,13 +66,13 @@ class DataEditCell(QtGui.QTableWidgetItem):
             else:
                 self.setText(self.node.data.get(self.field.name, ''))
             self.errorFlag = True
-        if self.field.useRichText:
+        if self.field.showRichTextInCell:
             self.doc.setHtml(self.text())
         else:
             self.doc.setPlainText(self.text())
 
 
-class DataEditDelegate(QtGui.QStyledItemDelegate):
+class DataEditDelegate(QStyledItemDelegate):
     """Class override for display and editing of DataEditCells.
     """
     def __init__(self, parent=None):
@@ -98,20 +102,19 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
             doc = cell.doc
             doc.setTextWidth(styleOption.rect.width())
             painter.translate(styleOption.rect.topLeft())
-            paintRect = QtCore.QRectF(0, 0, styleOption.rect.width(),
-                                      styleOption.rect.height())
+            paintRect = QRectF(0, 0, styleOption.rect.width(),
+                               styleOption.rect.height())
             painter.setClipRect(paintRect)
-            painter.fillRect(paintRect, QtGui.QApplication.palette().base())
-            painter.setPen(QtGui.QPen(QtGui.QApplication.palette().text(), 1))
+            painter.fillRect(paintRect, QApplication.palette().base())
+            painter.setPen(QPen(QApplication.palette().text(), 1))
             painter.drawRect(paintRect.adjusted(0, 0, -1, -1))
             doc.drawContents(painter)
             if cell.errorFlag:
-                path = QtGui.QPainterPath(QtCore.QPointF(0, 0))
+                path = QPainterPath(QPointF(0, 0))
                 path.lineTo(0, 10)
                 path.lineTo(10, 0)
                 path.closeSubpath()
-                painter.fillPath(path,
-                                 QtGui.QApplication.palette().highlight())
+                painter.fillPath(path, QApplication.palette().highlight())
             painter.restore()
         else:
             super().paint(painter, styleOption, modelIndex)
@@ -133,13 +136,13 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
             if size.height() > maxHeight:
                 size.setHeight(maxHeight)
             if cell.field.numLines > 1:
-                minDoc = QtGui.QTextDocument('\n' * (cell.field.numLines - 1))
-                minDoc.setDefaultFont(defaultFont)
+                minDoc = QTextDocument('\n' * (cell.field.numLines - 1))
+                minDoc.setDefaultFont(cell.doc.defaultFont())
                 minHeight = (minDoc.documentLayout().documentSize().toSize().
                              height())
                 if minHeight > size.height():
                     size.setHeight(minHeight)
-            return size + QtCore.QSize(0, 4)
+            return size + QSize(0, 4)
         return super().sizeHint(styleOption, modelIndex)
 
     def createEditor(self, parent, styleOption, modelIndex):
@@ -152,7 +155,7 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
         """
         cell = self.parent().item(modelIndex.row(), modelIndex.column())
         if isinstance(cell, DataEditCell):
-            editor = cell.field.editorClass(parent)
+            editor = getattr(dataeditors, cell.field.editorClassName)(parent)
             editor.setFont(cell.doc.defaultFont())
             if hasattr(editor, 'fieldRef'):
                 editor.fieldRef = cell.field
@@ -160,14 +163,15 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
                 editor.nodeRef = cell.node
             if cell.errorFlag:
                 editor.setErrorFlag()
-            self.parent().setFocusProxy(editor)
+            # self.parent().setFocusProxy(editor)
             editor.contentsChanged.connect(self.commitData)
             if hasattr(editor, 'inLinkSelectMode'):
-                editor.inLinkSelectMode.connect(self.parent().inLinkSelectMode)
+                editor.inLinkSelectMode.connect(self.parent().
+                                                changeInLinkSelectMode)
             if hasattr(editor, 'setLinkFromNode'):
                 self.parent().internalLinkSelected.connect(editor.
                                                            setLinkFromNode)
-            # viewport filter required to catch mouse events for undo limits
+            # viewport filter required to catch editor events
             try:
                 editor.viewport().installEventFilter(self)
             except AttributeError:
@@ -176,6 +180,7 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
                 except AttributeError:
                     pass
             self.lastEditor = editor
+            editor.setFocus()
             return editor
         return super().createEditor(parent, styleOption, modelIndex)
 
@@ -221,23 +226,16 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
                 numLines = newText.count('\n')
                 skipUndoAvail = numLines == self.prevNumLines
                 self.prevNumLines = numLines
-                undo.DataUndo(cell.node.modelRef.undoList, cell.node,
+                treeStructure = globalref.mainControl.activeControl.structure
+                undo.DataUndo(treeStructure.undoList, cell.node, False, False,
                               skipUndoAvail, cell.field.name)
                 try:
                     cell.node.setData(cell.field, newText)
                 except ValueError:
                     editor.setErrorFlag()
                 self.parent().nodeModified.emit(cell.node)
-                cell.titleCellRef.setText(cell.node.title())
-                cell.typeCellRef.setText(cell.node.formatName)
-                linkRefCollect = cell.node.modelRef.linkRefCollect
-                if (hasattr(editor, 'addedIntLinkFlag') and
-                    (editor.addedIntLinkFlag or
-                     linkRefCollect.linkCount(cell.node, cell.field.name))):
-                    linkRefCollect.searchForLinks(cell.node, cell.field.name)
-                    editor.addedIntLinkFlag = False
-                if cell.idCellRef:
-                    cell.idCellRef.setText(cell.node.uniqueId)
+                cell.titleCellRef.setText(cell.node.title(cell.spot))
+                cell.typeCellRef.setText(cell.node.formatRef.name)
                 editor.modified = False
         else:
             super().setModelData(editor, styleOption, modelIndex)
@@ -262,7 +260,7 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
             styleOption -- the data for styles and geometry  (not used)
             modelIndex -- the index of the cell (not used)
         """
-        if event.type() == QtCore.QEvent.MouseButtonPress:
+        if event.type() == QEvent.MouseButtonPress:
             self.editorClickPos = event.globalPos()
         return super().editorEvent(event, model, styleOption, modelIndex)
 
@@ -276,81 +274,79 @@ class DataEditDelegate(QtGui.QStyledItemDelegate):
             editor -- the editor that Qt installed a filter on
             event -- the key press event
         """
-        if event.type() == QtCore.QEvent.KeyPress:
+        if event.type() == QEvent.KeyPress:
             view = self.parent()
-            if (event.key() == QtCore.Qt.Key_Tab and
+            if (event.key() == Qt.Key_Tab and
                 view.currentRow() == view.rowCount() - 1):
                 view.focusOtherView.emit(True)
                 return True
-            if event.key() == QtCore.Qt.Key_Backtab:
-                startRow = 1
-                if globalref.genOptions.getValue('ShowUniqueID'):
-                    startRow = 2
-                if view.currentRow() == startRow:
-                    view.focusOtherView.emit(False)
-                    return True
-            if (event.modifiers() == QtCore.Qt.ControlModifier and
-                QtCore.Qt.Key_A <= event.key() <= QtCore.Qt.Key_Z):
-                key = QtGui.QKeySequence(event.modifiers() | event.key())
+            if (event.key() == Qt.Key_Backtab and view.currentRow() == 1):
+                view.focusOtherView.emit(False)
+                return True
+            if (event.modifiers() == Qt.ControlModifier and
+                Qt.Key_A <= event.key() <= Qt.Key_Z):
+                key = QKeySequence(event.modifiers() | event.key())
                 view.shortcutEntered.emit(key)
                 return True
-        if event.type() == QtCore.QEvent.MouseButtonPress:
-            self.prevNumLines = -1  # reset undo avail for mose cursor changes
-        if event.type() == QtCore.QEvent.FocusOut:
+        if event.type() == QEvent.MouseButtonPress:
+            self.prevNumLines = -1  # reset undo avail for mouse cursor changes
+        if event.type() == QEvent.FocusOut:
             self.prevNumLines = -1  # reset undo avail for any focus loss
-            if (event.reason() in (QtCore.Qt.MouseFocusReason,
-                                   QtCore.Qt.TabFocusReason,
-                                   QtCore.Qt.BacktabFocusReason) and
-                (not hasattr(editor, 'calendar') or
-                 not editor.calendar or not editor.calendar.isVisible()) and
+            if (event.reason() in (Qt.MouseFocusReason, Qt.TabFocusReason,
+                                   Qt.BacktabFocusReason)
+                and (not hasattr(editor, 'calendar') or
+                not editor.calendar or not editor.calendar.isVisible()) and
                 (not hasattr(editor, 'intLinkDialog') or
                  not editor.intLinkDialog or
                  not editor.intLinkDialog.isVisible())):
-                self.parent().setCurrentCell(-1, -1)
+                self.parent().endEditor()
                 return True
         return super().eventFilter(editor, event)
 
 
-class DataEditView(QtGui.QTableWidget):
+class DataEditView(QTableWidget):
     """Class override for the table-based data edit view.
     
     Sets view defaults and updates the content.
     """
-    nodeModified = QtCore.pyqtSignal(treenode.TreeNode)
-    inLinkSelectMode = QtCore.pyqtSignal(bool)
-    internalLinkSelected = QtCore.pyqtSignal(treenode.TreeNode)
-    focusOtherView = QtCore.pyqtSignal(bool)
-    shortcutEntered = QtCore.pyqtSignal(QtGui.QKeySequence)
-    def __init__(self, selectModel, allActions, isChildView=True,
+    nodeModified = pyqtSignal(treenode.TreeNode)
+    inLinkSelectMode = pyqtSignal(bool)
+    internalLinkSelected = pyqtSignal(treenode.TreeNode)
+    focusOtherView = pyqtSignal(bool)
+    hoverFocusActive = pyqtSignal()
+    shortcutEntered = pyqtSignal(QKeySequence)
+    def __init__(self, treeView, allActions, isChildView=True,
                  parent=None):
         """Initialize the data edit view default view settings.
 
         Arguments:
-            selectModel - the tree view's selection model
+            treeView - the tree view, needed for the current selection model
             allActions -- a dict containing actions for the editor context menu
             isChildView -- shows selected nodes if false, child nodes if true
             parent -- the parent main window
         """
         super().__init__(0, 2, parent)
-        self.selectModel = selectModel
+        self.treeView = treeView
         self.allActions = allActions
         self.isChildView = isChildView
-        self.hideChildView = not globalref.genOptions.getValue('ShowChildPane')
+        self.hideChildView = not globalref.genOptions['InitShowChildPane']
+        self.prevHoverCell = None
+        self.inLinkSelectActive = False
         self.setAcceptDrops(True)
+        self.setMouseTracking(globalref.genOptions['EditorOnHover'])
         self.horizontalHeader().hide()
         self.verticalHeader().hide()
-        self.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
-        self.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
-        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
+        self.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
+        self.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
+        self.setSelectionMode(QAbstractItemView.SingleSelection)
         self.setItemDelegate(DataEditDelegate(self))
-        self.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
+        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
         self.setShowGrid(False)
-        self.setTabKeyNavigation(False)
         pal = self.palette()
-        pal.setBrush(QtGui.QPalette.Base,
-                     QtGui.QApplication.palette().window())
-        pal.setBrush(QtGui.QPalette.Text,
-                     QtGui.QApplication.palette().windowText())
+        pal.setBrush(QPalette.Base,
+                     QApplication.palette().window())
+        pal.setBrush(QPalette.Text,
+                     QApplication.palette().windowText())
         self.setPalette(pal)
         self.currentItemChanged.connect(self.moveEditor)
 
@@ -359,61 +355,63 @@ class DataEditView(QtGui.QTableWidget):
 
         Avoids update if view is not visible or has zero height or width.
         """
-        selNodes = self.selectModel.selectedNodes()
-        if self.isChildView and (len(selNodes) != 1 or self.hideChildView or
-                                 not selNodes[0].childList):
+        selSpots = self.treeView.selectionModel().selectedSpots()
+        if self.isChildView:
+            if (len(selSpots) > 1 or self.hideChildView or
+                (selSpots and not selSpots[0].nodeRef.childList)):
+                self.hide()
+                return
+            if not selSpots:
+                # use top node childList from tree structure
+                selSpots = [globalref.mainControl.activeControl.structure.
+                            structSpot()]
+        elif not selSpots:
             self.hide()
-        else:
-            self.show()
+            return
+        self.show()
         if not self.isVisible() or self.height() == 0 or self.width() == 0:
             return
         if self.isChildView:
-            selNodes = selNodes[0].childList
+            selSpots = selSpots[0].childSpots()
         self.clear()
-        self.clearSpans()
-        if selNodes:
+        if selSpots:
             self.hide() # 2nd update very slow if shown during update
             self.setRowCount(100000)
             rowNum = -2
-            for node in selNodes:
-                rowNum = self.addNodeData(node, rowNum + 2)
+            for spot in selSpots:
+                rowNum = self.addNodeData(spot, rowNum + 2)
             self.setRowCount(rowNum + 1)
             self.adjustSizes()
+            self.scrollToTop()
             self.show()
 
-    def addNodeData(self, node, startRow):
+    def addNodeData(self, spot, startRow):
         """Populate the view with the data from the given node.
 
         Returns the last row number used.
         Arguments:
-            node -- the node to add
+            spot -- the spot to add
             startRow -- the row offset
         """
-        typeCell = self.createInactiveCell(node.formatName)
+        node = spot.nodeRef
+        formatName = node.formatRef.name
+        typeCell = self.createInactiveCell(formatName)
         self.setItem(startRow, 0, typeCell)
-        titleCell = self.createInactiveCell(node.title())
+        titleCell = self.createInactiveCell(node.title(spot))
         self.setItem(startRow, 1, titleCell)
-        idCell = None
-        if globalref.genOptions.getValue('ShowUniqueID'):
-            startRow += 1
-            self.setItem(startRow, 0, self.createInactiveCell(_('Unique ID'),
-                                                       QtCore.Qt.AlignRight |
-                                                       QtCore.Qt.AlignVCenter))
-            idCell = self.createInactiveCell(node.uniqueId)
-            self.setItem(startRow, 1, idCell)
-        fields = node.nodeFormat().fields()
-        if not globalref.genOptions.getValue('EditNumbering'):
+        fields = node.formatRef.fields()
+        if not globalref.genOptions['EditNumbering']:
             fields = [field for field in fields
                       if field.typeName != 'Numbering']
-        if not globalref.genOptions.getValue('ShowMath'):
+        if not globalref.genOptions['ShowMath']:
             fields = [field for field in fields
                       if field.typeName != 'Math']
         for row, field in enumerate(fields, startRow + 1):
             self.setItem(row, 0, self.createInactiveCell(field.name,
-                                                       QtCore.Qt.AlignRight |
-                                                       QtCore.Qt.AlignVCenter))
-            self.setItem(row, 1, DataEditCell(node, field, titleCell, typeCell,
-                                              idCell))
+                                                         Qt.AlignRight |
+                                                         Qt.AlignVCenter))
+            self.setItem(row, 1, DataEditCell(spot, field, titleCell,
+                                              typeCell))
         self.setItem(row + 1, 0, self.createInactiveCell(''))
         self.setItem(row + 1, 1, self.createInactiveCell(''))
         return row
@@ -423,14 +421,20 @@ class DataEditView(QtGui.QTableWidget):
         """
         if not self.isVisible() or self.height() == 0 or self.width() == 0:
             return
-        selNodes = self.selectModel.selectedNodes()
+        selSpots = self.treeView.selectionModel().selectedSpots()
         if self.isChildView:
-            selNodes = selNodes[0].childList
+            if not selSpots:
+                # use top node childList from tree structure
+                selSpots = [globalref.mainControl.activeControl.structure.
+                            structSpot()]
+            selSpots = selSpots[0].childSpots()
+        elif not selSpots:
+            return
         rowNum = -2
-        for node in selNodes:
-            rowNum = self.refreshNodeData(node, rowNum + 2)
+        for spot in selSpots:
+            rowNum = self.refreshNodeData(spot, rowNum + 2)
 
-    def refreshNodeData(self, node, startRow):
+    def refreshNodeData(self, spot, startRow):
         """Refresh the data in active cells for this node.
 
         Returns the last row number used.
@@ -438,20 +442,18 @@ class DataEditView(QtGui.QTableWidget):
             node -- the node to add
             startRow -- the row offset
         """
-        self.item(startRow, 1).setText(node.title())
-        if globalref.genOptions.getValue('ShowUniqueID'):
-            startRow += 1
-            self.item(startRow, 1).setText(node.uniqueId)
-        fields = node.nodeFormat().fields()
-        if not globalref.genOptions.getValue('EditNumbering'):
+        node = spot.nodeRef
+        self.item(startRow, 1).setText(node.title(spot))
+        fields = node.formatRef.fields()
+        if not globalref.genOptions['EditNumbering']:
             fields = [field for field in fields
                       if field.typeName != 'Numbering']
-        if not globalref.genOptions.getValue('ShowMath'):
+        if not globalref.genOptions['ShowMath']:
             fields = [field for field in fields
                       if field.typeName != 'Math']
         for row, field in enumerate(fields, startRow + 1):
             cell = self.item(row, 1)
-            if not cell.isSelected() or not self.hasFocus():
+            if not cell.isSelected():
                 cell.updateText()
         return row
 
@@ -463,8 +465,8 @@ class DataEditView(QtGui.QTableWidget):
             text -- the initial text string for the cell
             alignment -- the text alignment QT constant (None for default)
         """
-        cell = QtGui.QTableWidgetItem(text)
-        cell.setFlags(QtCore.Qt.NoItemFlags)
+        cell = QTableWidgetItem(text)
+        cell.setFlags(Qt.NoItemFlags)
         if alignment:
             cell.setTextAlignment(alignment)
         return cell
@@ -483,6 +485,11 @@ class DataEditView(QtGui.QTableWidget):
         if newCell:
             self.openPersistentEditor(newCell)
 
+    def endEditor(self):
+        """End persistent editors by changing active cells.
+        """
+        self.setCurrentCell(-1, -1)
+
     def setFont(self, font):
         """Override to avoid setting fonts of inactive cells.
 
@@ -492,6 +499,25 @@ class DataEditView(QtGui.QTableWidget):
         global defaultFont
         defaultFont = font
 
+    def changeInLinkSelectMode(self, active=True):
+        """Change the internal link select mode.
+
+        Changes the internal variable (controlling hover) and signals the tree.
+        Arguments:
+            active -- if True, starts the mode, o/w ends
+        """
+        self.inLinkSelectActive = active
+        self.inLinkSelectMode.emit(active)
+
+    def updateInLinkSelectMode(self, active=True):
+        """Update the internal link select mode.
+
+        Updates the internal variable (controlling hover).
+        Arguments:
+            active -- if True, starts the mode, o/w ends
+        """
+        self.inLinkSelectActive = active
+
     def highlightSearch(self, wordList=None, regExpList=None):
         """Highlight any found search terms.
 
@@ -499,28 +525,29 @@ class DataEditView(QtGui.QTableWidget):
             wordList -- list of words to highlight
             regExpList -- a list of regular expression objects to highlight
         """
-        backColor = self.palette().brush(QtGui.QPalette.Active,
-                                         QtGui.QPalette.Highlight)
-        foreColor = self.palette().brush(QtGui.QPalette.Active,
-                                         QtGui.QPalette.HighlightedText)
-        charFormat = QtGui.QTextCharFormat()
+        backColor = self.palette().brush(QPalette.Active,
+                                         QPalette.Highlight)
+        foreColor = self.palette().brush(QPalette.Active,
+                                         QPalette.HighlightedText)
+        charFormat = QTextCharFormat()
         charFormat.setBackground(backColor)
         charFormat.setForeground(foreColor)
-        node = self.selectModel.selectedNodes()[0]
+        spot = self.treeView.selectionModel().selectedSpots()[0]
         if wordList is None:
             wordList = []
         if regExpList is None:
             regExpList = []
         for regExp in regExpList:
-            for match in regExp.finditer('\n'.join(node.formatOutput())):
+            for match in regExp.finditer('\n'.join(spot.nodeRef.
+                                                   output(spotRef=spot))):
                 matchText = match.group().lower()
                 if matchText not in wordList:
                     wordList.append(matchText)
         cells = []
         completedCells = []
         for word in wordList:
-            cells.extend(self.findItems(word, QtCore.Qt.MatchFixedString |
-                                        QtCore.Qt.MatchContains))
+            cells.extend(self.findItems(word, Qt.MatchFixedString |
+                                        Qt.MatchContains))
         for cell in cells:
             if hasattr(cell, 'doc') and cell not in completedCells:
                 highlighter = SearchHighlighter(wordList, charFormat, cell.doc)
@@ -536,16 +563,14 @@ class DataEditView(QtGui.QTableWidget):
             cellNum -- the vertical position (field number) of the cell
             skipMatches -- number of previous matches to skip in this field
         """
-        backColor = self.palette().brush(QtGui.QPalette.Active,
-                                         QtGui.QPalette.Highlight)
-        foreColor = self.palette().brush(QtGui.QPalette.Active,
-                                         QtGui.QPalette.HighlightedText)
-        charFormat = QtGui.QTextCharFormat()
+        backColor = self.palette().brush(QPalette.Active,
+                                         QPalette.Highlight)
+        foreColor = self.palette().brush(QPalette.Active,
+                                         QPalette.HighlightedText)
+        charFormat = QTextCharFormat()
         charFormat.setBackground(backColor)
         charFormat.setForeground(foreColor)
         cellNum += 1    # skip title line
-        if globalref.genOptions.getValue('ShowUniqueID'):
-            cellNum += 1
         cell = self.item(cellNum, 1)
         highlighter = MatchHighlighter(cell.doc, charFormat, searchText,
                                        regExpObj, skipMatches)
@@ -567,24 +592,19 @@ class DataEditView(QtGui.QTableWidget):
         Arguments:
             event -- the focus in event
         """
-        if event.reason() == QtCore.Qt.TabFocusReason:
+        if event.reason() == Qt.TabFocusReason:
             for row in range(self.rowCount()):
                 cell = self.item(row, 1)
                 if hasattr(cell, 'doc'):
                     self.setCurrentItem(cell)
-                    self.setFocus()
-                    event.accept()
-                    return
-        elif event.reason() == QtCore.Qt.BacktabFocusReason:
+                    break
+        elif event.reason() == Qt.BacktabFocusReason:
             for row in range(self.rowCount() - 1, -1, -1):
                 cell = self.item(row, 1)
                 if hasattr(cell, 'doc'):
                     self.setCurrentItem(cell)
-                    self.setFocus()
-                    event.accept()
-                    return
-        else:
-            super().focusInEvent(event)
+                    break
+        super().focusInEvent(event)
 
     def resizeEvent(self, event):
         """Update view if was collaped by splitter.
@@ -612,7 +632,7 @@ class DataEditView(QtGui.QTableWidget):
         """
         cell = self.itemAt(event.pos())
         if (isinstance(cell, DataEditCell) and
-            cell.field.editorClass.dragLinkEnabled):
+            getattr(dataeditors, cell.field.editorClassName).dragLinkEnabled):
             event.accept()
         else:
             event.ignore()
@@ -637,36 +657,64 @@ class DataEditView(QtGui.QTableWidget):
         Arguments:
             event -- the mouse event
         """
-        if (event.button() == QtCore.Qt.LeftButton and
-            event.modifiers() == QtCore.Qt.ControlModifier):
+        if (event.button() == Qt.LeftButton and
+            event.modifiers() == Qt.ControlModifier):
             cell = self.itemAt(event.pos())
             if cell and isinstance(cell, DataEditCell):
                 xOffest = (event.pos().x() -
                            self.columnViewportPosition(cell.column()))
                 yOffset = (event.pos().y() -
                            self.rowViewportPosition(cell.row()))
-                pt = QtCore.QPointF(xOffest, yOffset)
-                pos = cell.doc.documentLayout().hitTest(pt, QtCore.Qt.ExactHit)
+                pt = QPointF(xOffest, yOffset)
+                pos = cell.doc.documentLayout().hitTest(pt, Qt.ExactHit)
                 if pos >= 0:
-                    cursor = QtGui.QTextCursor(cell.doc)
+                    cursor = QTextCursor(cell.doc)
                     cursor.setPosition(pos)
                     address = cursor.charFormat().anchorHref()
                     if address:
                         if address.startswith('#'):
-                            self.selectModel.selectNodeById(address[1:])
+                            (self.treeView.selectionModel().
+                             selectNodeById(address[1:]))
                         else:     # check for relative path
                             if urltools.isRelative(address):
                                 defaultPath = (globalref.mainControl.
-                                               defaultFilePath(True))
+                                               defaultPathObj(True))
                                 address = urltools.toAbsolute(address,
-                                                              defaultPath)
+                                                              str(defaultPath))
                             dataeditors.openExtUrl(address)
             event.accept()
         else:
             super().mousePressEvent(event)
 
+    def mouseMoveEvent(self, event):
+        """Handle mouse move event to create editors on hover.
 
-class SearchHighlighter(QtGui.QSyntaxHighlighter):
+        Arguments:
+            event -- the mouse event
+        """
+        cell = self.itemAt(event.pos())
+        if cell and hasattr(cell, 'doc'):
+            oldCell = self.currentItem()
+            if (cell != oldCell and cell != self.prevHoverCell and
+                not self.inLinkSelectActive):
+                self.prevHoverCell = cell
+                self.hoverFocusActive.emit()
+                self.setFocus()
+                if oldCell and hasattr(oldCell, 'doc'):
+                    # these lines result in two calls to moveEditor, but seems
+                    # to be necessary to avoid race that leaves stray editors
+                    self.moveEditor(None, oldCell)
+                    self.setCurrentItem(None)
+                try:
+                    self.setCurrentItem(cell)
+                except RuntimeError:
+                    # catch error if view updates due to node rename ending
+                    pass
+        else:
+            self.prevHoverCell = None
+
+
+class SearchHighlighter(QSyntaxHighlighter):
     """Class override to highlight search terms in cell text.
 
     Used to highlight search words from a list.
@@ -696,7 +744,7 @@ class SearchHighlighter(QtGui.QSyntaxHighlighter):
                 pos = text.lower().find(word, pos + len(word))
 
 
-class MatchHighlighter(QtGui.QSyntaxHighlighter):
+class MatchHighlighter(QSyntaxHighlighter):
     """Class override to highlight a specific search result in cell text.
 
     Used to highlight a text or reg exp match.
@@ -733,6 +781,6 @@ class MatchHighlighter(QtGui.QSyntaxHighlighter):
             else:
                 match = self.regExpObj.search(text, pos)
                 pos = match.start() if match else -1
-                matchLen = len(match.group())
+                matchLen = len(match.group()) if match else 0
         if pos >= 0:
             self.setFormat(pos, matchLen, self.charFormat)
