@@ -4,7 +4,7 @@
 # globalref.py, provides a module for access to a few global variables
 #
 # TreeLine, an information storage program
-# Copyright (C) 2014, Douglas W. Bell
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -26,14 +26,20 @@ treeIcons = None
 localTextEncoding = ''
 lang = ''
 
-fileFilters = {'trl': '{} (*.trl *.xml)'.format(_('TreeLine Files')),
-               'trlgz': '{} (*.trl *.trl.gz)'.
-                        format(_('TreeLine Files - Compressed')),
-               'trlenc': '{} (*.trl)'.format(_('TreeLine Files - Encrypted')),
+fileFilters = {'trlnopen': '{} (*.trln *.trln.gz *.trl)'.
+                           format(_('All TreeLine Files')),
+               'trlnv3': '{} (*.trln *.trln.gz)'.format(_('TreeLine Files')),
+               'trlnsave': '{} (*.trln)'.format(_('TreeLine Files')),
+               'trlngz': '{} (*.trln *.trln.gz)'.
+                         format(_('TreeLine Files - Compressed')),
+               'trlnenc': '{} (*.trln)'.
+                          format(_('TreeLine Files - Encrypted')),
+               'trl': '{} (*.trl *.xml)'.format(_('Old TreeLine Files')),
                'all': '{} (*)'.format(_('All Files')),
                'html': '{} (*.html *.htm)'.format(_('HTML Files')),
                'txt': '{} (*.txt)'.format(_('Text Files')),
                'xml': '{} (*.xml)'.format(_('XML Files')),
+               'csv': '{} (*.csv)'.format(_('CSV (Comma Delimited) Files')),
                'odt': '{} (*.odt)'.format(_('ODF Text Files')),
                'hjt': '{} (*.hjt)'.format(_('Treepad Files')),
                'pdf': '{} (*.pdf)'.format(_('PDF Files'))}
