@@ -3,7 +3,7 @@
 #****************************************************************************
 # helpview.py, provides a window for viewing an html help file
 #
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -11,36 +11,39 @@
 # but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #*****************************************************************************
 
-import os.path
-import sys
-from PyQt4 import QtCore, QtGui
+from PyQt5.QtCore import QUrl, Qt
+from PyQt5.QtGui import QTextDocument
+from PyQt5.QtWidgets import (QAction, QLabel, QLineEdit, QMainWindow, QMenu,
+                             QStatusBar, QTextBrowser)
 import dataeditors
 
 
-class HelpView(QtGui.QMainWindow):
+class HelpView(QMainWindow):
     """Main window for viewing an html help file.
     """
-    def __init__(self, path, caption, icons, parent=None):
+    def __init__(self, pathObj, caption, icons, parent=None):
         """Helpview initialize with text.
+
+        Arguments:
+            pathObj -- a path object for the help file
+            caption -- the window caption
+            icons -- dict of view icons
         """
-        QtGui.QMainWindow.__init__(self, parent)
-        self.setAttribute(QtCore.Qt.WA_QuitOnClose, False)
-        self.setWindowFlags(QtCore.Qt.Window)
-        self.setStatusBar(QtGui.QStatusBar())
+        QMainWindow.__init__(self, parent)
+        self.setAttribute(Qt.WA_QuitOnClose, False)
+        self.setWindowFlags(Qt.Window)
+        self.setStatusBar(QStatusBar())
         self.textView = HelpViewer(self)
         self.setCentralWidget(self.textView)
-        path = os.path.abspath(path)
-        if sys.platform.startswith('win'):
-            path = path.replace('\\', '/')
-        self.textView.setSearchPaths([os.path.dirname(path)])
-        self.textView.setSource(QtCore.QUrl('file:///{0}'.format(path)))
+        self.textView.setSearchPaths([str(pathObj.parent)])
+        self.textView.setSource(QUrl(pathObj.as_uri()))
         self.resize(520, 440)
         self.setWindowTitle(caption)
         tools = self.addToolBar(_('Tools'))
-        self.menu = QtGui.QMenu(self.textView)
+        self.menu = QMenu(self.textView)
         self.textView.highlighted[str].connect(self.showLink)
 
-        backAct = QtGui.QAction(_('&Back'), self)
+        backAct = QAction(_('&Back'), self)
         backAct.setIcon(icons['helpback'])
         tools.addAction(backAct)
         self.menu.addAction(backAct)
@@ -48,7 +51,7 @@ class HelpView(QtGui.QMainWindow):
         backAct.setEnabled(False)
         self.textView.backwardAvailable.connect(backAct.setEnabled)
 
-        forwardAct = QtGui.QAction(_('&Forward'), self)
+        forwardAct = QAction(_('&Forward'), self)
         forwardAct.setIcon(icons['helpforward'])
         tools.addAction(forwardAct)
         self.menu.addAction(forwardAct)
@@ -56,7 +59,7 @@ class HelpView(QtGui.QMainWindow):
         forwardAct.setEnabled(False)
         self.textView.forwardAvailable.connect(forwardAct.setEnabled)
 
-        homeAct = QtGui.QAction(_('&Home'), self)
+        homeAct = QAction(_('&Home'), self)
         homeAct.setIcon(icons['helphome'])
         tools.addAction(homeAct)
         self.menu.addAction(homeAct)
@@ -64,21 +67,21 @@ class HelpView(QtGui.QMainWindow):
 
         tools.addSeparator()
         tools.addSeparator()
-        findLabel = QtGui.QLabel(_(' Find: '), self)
+        findLabel = QLabel(_(' Find: '), self)
         tools.addWidget(findLabel)
-        self.findEdit = QtGui.QLineEdit(self)
+        self.findEdit = QLineEdit(self)
         tools.addWidget(self.findEdit)
         self.findEdit.textEdited.connect(self.findTextChanged)
         self.findEdit.returnPressed.connect(self.findNext)
 
-        self.findPreviousAct = QtGui.QAction(_('Find &Previous'), self)
+        self.findPreviousAct = QAction(_('Find &Previous'), self)
         self.findPreviousAct.setIcon(icons['helpprevious'])
         tools.addAction(self.findPreviousAct)
         self.menu.addAction(self.findPreviousAct)
         self.findPreviousAct.triggered.connect(self.findPrevious)
         self.findPreviousAct.setEnabled(False)
 
-        self.findNextAct = QtGui.QAction(_('Find &Next'), self)
+        self.findNextAct = QAction(_('Find &Next'), self)
         self.findNextAct.setIcon(icons['helpnext'])
         tools.addAction(self.findNextAct)
         self.menu.addAction(self.findNextAct)
@@ -87,11 +90,17 @@ class HelpView(QtGui.QMainWindow):
 
     def showLink(self, text):
         """Send link text to the statusbar.
+
+        Arguments:
+            text -- link text to show
         """
         self.statusBar().showMessage(text)
 
     def findTextChanged(self, text):
         """Update find controls based on text in text edit.
+
+        Arguments:
+            text -- the search text
         """
         self.findPreviousAct.setEnabled(len(text) > 0)
         self.findNextAct.setEnabled(len(text) > 0)
@@ -100,7 +109,7 @@ class HelpView(QtGui.QMainWindow):
         """Command to find the previous string.
         """
         if self.textView.find(self.findEdit.text(),
-                              QtGui.QTextDocument.FindBackward):
+                              QTextDocument.FindBackward):
             self.statusBar().clearMessage()
         else:
             self.statusBar().showMessage(_('Text string not found'))
@@ -114,22 +123,33 @@ class HelpView(QtGui.QMainWindow):
             self.statusBar().showMessage(_('Text string not found'))
 
 
-class HelpViewer(QtGui.QTextBrowser):
+class HelpViewer(QTextBrowser):
     """Shows an html help file.
     """
     def __init__(self, parent=None):
-        QtGui.QTextBrowser.__init__(self, parent)
+        """Initialize the viewer.
+
+        Arguments:
+            parent -- the parent widget, if given
+        """
+        QTextBrowser.__init__(self, parent)
 
     def setSource(self, url):
         """Called when user clicks on a URL.
+
+        Arguments:
+            url -- the clicked on QUrl
         """
         name = url.toString()
         if name.startswith('http'):
             dataeditors.openExtUrl(name)
         else:
-            QtGui.QTextBrowser.setSource(self, QtCore.QUrl(name))
+            QTextBrowser.setSource(self, QUrl(name))
 
     def contextMenuEvent(self, event):
         """Init popup menu on right click"".
+
+        Arguments:
+            event -- the menu event
         """
         self.parentWidget().menu.exec_(event.globalPos())
