@@ -4,7 +4,7 @@
 # matheval.py, provides a safe eval of mathematical expressions
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -14,6 +14,7 @@
 
 import re
 import ast
+import enum
 import builtins
 import gennumber
 from math import *
@@ -213,7 +214,7 @@ class MathEquation:
             eqnNode -- the node containing the equation to evaluate
             zeroValue -- the value to use for blanks
         """
-        zeroBlanks = eqnNode.modelRef.mathZeroBlanks
+        zeroBlanks = eqnNode.treeStructureRef().mathZeroBlanks
         inputs = [ref.referenceValue(eqnNode, zeroBlanks) for ref in
                   self.fieldRefs]
         if not zeroBlanks and None in inputs:
@@ -222,7 +223,7 @@ class MathEquation:
         eqn = self.formattedEqnText.format(*inputs)
         try:
             return eval(eqn)
-        except (TypeError, ZeroDivisionError) as err:
+        except Exception as err:
             raise ValueError(err)
 
     def parseEquation(self, eqnText):
@@ -253,7 +254,7 @@ class MathEquation:
 
 
 # recursive equation ref eval directions
-optional, upward, downward = range(3)
+EvalDir = enum.IntEnum('EvalDir', 'optional upward downward')
 
 
 class EquationFieldRef:
@@ -263,7 +264,7 @@ class EquationFieldRef:
     """
     tagPrefix = ''
     testValue = 1
-    evalDirection = optional
+    evalDirection = EvalDir.optional
     def __init__(self, fieldName):
         """Initialize the field references.
 
@@ -285,7 +286,7 @@ class EquationFieldRef:
             zeroValue -- the value to use for blanks
         """
         try:
-            return (eqnNode.nodeFormat().fieldDict[self.fieldName].
+            return (eqnNode.formatRef.fieldDict[self.fieldName].
                     mathValue(eqnNode, zeroBlanks))
         except KeyError:
             return zeroValue if zeroBlanks else None
@@ -296,7 +297,7 @@ class EquationFieldRef:
         Arguments:
             refNode -- the node containing the referenced field
         """
-        if refNode.formatName == self.eqnNodeTypeName:
+        if refNode.formatRef.name == self.eqnNodeTypeName:
             return [refNode]
         return []
 
@@ -306,7 +307,7 @@ class EquationParentRef(EquationFieldRef):
     """
     tagPrefix = '*'
     testValue = 1
-    evalDirection = downward
+    evalDirection = EvalDir.downward
 
     def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
         """Return the parent field value referenced from a given node.
@@ -318,11 +319,11 @@ class EquationParentRef(EquationFieldRef):
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
         """
-        node = eqnNode.parent
-        if not node:
+        node = eqnNode.spotByNumber(0).parentSpot.nodeRef
+        if not node.formatRef:
             return zeroValue if zeroBlanks else None
         try:
-            return (node.nodeFormat().fieldDict[self.fieldName].
+            return (node.formatRef.fieldDict[self.fieldName].
                     mathValue(node, zeroBlanks))
         except KeyError:
             return zeroValue if zeroBlanks else None
@@ -334,7 +335,7 @@ class EquationParentRef(EquationFieldRef):
             refNode -- the node containing the referenced field
         """
         return [node for node in refNode.childList if
-                node.formatName == self.eqnNodeTypeName]
+                node.formatRef.name == self.eqnNodeTypeName]
 
 
 class EquationRootRef(EquationFieldRef):
@@ -342,7 +343,7 @@ class EquationRootRef(EquationFieldRef):
     """
     tagPrefix = '$'
     testValue = 1
-    evalDirection = downward
+    evalDirection = EvalDir.downward
 
     def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
         """Return the root field value referenced from a given node.
@@ -354,9 +355,9 @@ class EquationRootRef(EquationFieldRef):
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
         """
-        node = eqnNode.modelRef.root
+        node = eqnNode.spotByNumber(0).spotChain()[0].nodeRef
         try:
-            return (node.nodeFormat().fieldDict[self.fieldName].
+            return (node.formatRef.fieldDict[self.fieldName].
                     mathValue(node, zeroBlanks))
         except KeyError:
             return zeroValue if zeroBlanks else None
@@ -367,10 +368,14 @@ class EquationRootRef(EquationFieldRef):
         Arguments:
             refNode -- the node containing the referenced field
         """
-        if refNode.parent:
+        if 1 not in {len(spot.spotChain()) for spot in refNode.spotRefs}:
+            # not a root node
             return []
-        return [node for node in refNode.selectiveDescendantGen() if
-                node.formatName == self.eqnNodeTypeName]
+        refs = [node for node in refNode.descendantGen() if
+                node.formatRef.name == self.eqnNodeTypeName]
+        if refs[0] is refNode:
+            refs = refs[1:]
+        return refs
 
 
 class EquationChildRef(EquationFieldRef):
@@ -378,7 +383,7 @@ class EquationChildRef(EquationFieldRef):
     """
     tagPrefix = '&'
     testValue = [1]
-    evalDirection = upward
+    evalDirection = EvalDir.upward
 
     def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
         """Return a list with child field values referenced from a given node.
@@ -393,7 +398,7 @@ class EquationChildRef(EquationFieldRef):
         result = []
         for node in eqnNode.childList:
             try:
-                num = (node.nodeFormat().fieldDict[self.fieldName].
+                num = (node.formatRef.fieldDict[self.fieldName].
                        mathValue(node, zeroBlanks))
                 if num == None:
                     return None
@@ -411,8 +416,8 @@ class EquationChildRef(EquationFieldRef):
         Arguments:
             refNode -- the node containing the referenced field
         """
-        node = refNode.parent
-        if node and node.formatName == self.eqnNodeTypeName:
+        node = refNode.spotByNumber(0).parentSpot.nodeRef
+        if node.formatRef and node.formatRef.name == self.eqnNodeTypeName:
             return [node]
         return []
 
@@ -422,7 +427,7 @@ class EquationChildCountRef(EquationFieldRef):
     """
     tagPrefix = '#'
     testValue = 1
-    evalDirection = optional
+    evalDirection = EvalDir.optional
 
     def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
         """Return the child count referenced from the given node.
@@ -440,8 +445,8 @@ class EquationChildCountRef(EquationFieldRef):
         Arguments:
             refNode -- the node containing the referenced field
         """
-        node = refNode.parent
-        if node and node.formatName == self.eqnNodeTypeName:
+        node = refNode.spotByNumber(0).parentSpot.nodeRef
+        if node and node.formatRef.name == self.eqnNodeTypeName:
             return [node]
         return []
 
@@ -462,7 +467,7 @@ class RecursiveEqnRef:
         self.eqnTypeName = eqnTypeName
         self.eqnField = eqnField
         self.evalSequence = 0
-        self.evalDirection = optional
+        self.evalDirection = EvalDir.optional
 
     def setPriorities(self, visitedFields=None):
         """Recursively set sequence and direction for evaluation.
@@ -480,7 +485,7 @@ class RecursiveEqnRef:
         for fieldRef in self.eqnField.equation.fieldRefs:
             if (fieldRef.fieldName in visitedFields and
                 (self.eqnField.name != fieldRef.fieldName or
-                 fieldRef.evalDirection == optional)):
+                 fieldRef.evalDirection == EvalDir.optional)):
                 raise CircularMathError()
             for eqnRef in self.recursiveRefDict.get(fieldRef.fieldName, []):
                 eqnRef.setPriorities(visitedFields)
@@ -488,7 +493,7 @@ class RecursiveEqnRef:
                     self.evalDirection = fieldRef.evalDirection
                     self.evalSequence = eqnRef.evalSequence
                     if (self.evalDirection != eqnRef.evalDirection or
-                        self.evalDirection == optional):
+                        self.evalDirection == EvalDir.optional):
                         self.evalSequence += 1
 
     def __lt__(self, other):
