@@ -4,7 +4,7 @@
 # nodeformat.py, provides a class to handle node format objects
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2018, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -16,22 +16,18 @@ import re
 import collections
 import os.path
 import sys
-import stat
 import copy
 import operator
+import datetime
 import xml.sax.saxutils
 if not sys.platform.startswith('win'):
     import pwd
-from PyQt4 import QtCore, QtGui
 import fieldformat
 import conditional
-import imports
 
 
 defaultFieldName = _('Name')
-uniqueIdFieldName = 'Node_Unique_ID'
 _defaultOutputSeparator = ', '
-_lineAttrRe = re.compile(r'line\d+$')
 _fieldSplitRe = re.compile(r'({\*(?:\**|\?|!|&|#)[\w_\-.]+\*})')
 _fieldPartRe = re.compile(r'{\*(\**|\?|!|&|#)([\w_\-.]+)\*}')
 _endTagRe = re.compile(r'.*(<br[ /]*?>|<BR[ /]*?>|<hr[ /]*?>|<HR[ /]*?>)$')
@@ -39,60 +35,120 @@ _levelFieldRe = re.compile(r'[^0-9]+([0-9]+)$')
 
 class NodeFormat:
     """Class to handle node format info
-    
+
     Stores node field lists and line formatting.
     Provides methods to return formatted data.
     """
-    def __init__(self, name, parentFormats, attrs=None, addDefaultField=False):
-        """Initialize a tree node.
+    def __init__(self, name, parentFormats, formatData=None,
+                 addDefaultField=False):
+        """Initialize a tree format.
 
         Arguments:
             name -- the type name string
             parentFormats -- a ref to TreeFormats class for outside field refs
-            attrs -- the attributes that define this type's format
+            formatData -- the JSON dict for this format
             addDefaultField -- if true, adds a default initial field
         """
         self.name = name
         self.parentFormats = parentFormats
-        self.idField = None
-        if not attrs:
-            attrs = {}
-        self.spaceBetween = attrs.get('spacebetween', 'y').startswith('y')
-        self.formatHtml = attrs.get('formathtml', '').startswith('y')
-        self.useBullets = attrs.get('bullets', '').startswith('y')
-        self.useTables = attrs.get('tables', '').startswith('y')
-        self.childType = attrs.get('childtype', '')
-        self.genericType = attrs.get('generic', '')
-        self.conditional = conditional.Conditional(attrs.get('condition', ''))
-        self.iconName = attrs.get('icon', '')
-        self.outputSeparator = attrs.get('outputsep', _defaultOutputSeparator)
         self.savedConditionText = {}
-        for key in attrs.keys():
-            if key.startswith('cond-'):
-                self.savedConditionText[key[5:]] = attrs[key]
-        self.fieldDict = collections.OrderedDict()
-        self.derivedTypes = []
+        self.conditional = None
+        self.childTypeLimit = set()
+        self.readFormat(formatData)
         self.siblingPrefix = ''
         self.siblingSuffix = ''
+        self.derivedTypes = []
+        self.origOutputLines = [] # lines without bullet or table modifications
         self.sortFields = []   # temporary storage while sorting
         if addDefaultField:
             self.addFieldIfNew(defaultFieldName)
-            attrs['line0'] = '{{*{0}*}}'.format(defaultFieldName)
-            attrs['line1'] = '{{*{0}*}}'.format(defaultFieldName)
-        self.lineList = [['']]
-        self.origLineList = []  # lines without bullet or table modifications
-        lineNums = sorted([int(key[4:]) for key in attrs.keys()
-                           if _lineAttrRe.match(key)])
-        if lineNums:
-            self.lineList = [[attrs['line{0}'.format(keyNum)]] for keyNum in
-                             lineNums]
-        if addDefaultField:
-            self.updateLineParsing()
+            self.titleLine = ['{{*{0}*}}'.format(defaultFieldName)]
+            self.outputLines = [['{{*{0}*}}'.format(defaultFieldName)]]
+        self.updateLineParsing()
         if self.useBullets:
             self.addBullets()
         if self.useTables:
             self.addTables()
 
+    def readFormat(self, formatData=None):
+        """Read JSON format data into this format.
+
+        Arguments:
+            formatData -- JSON dict for this format (None for default settings)
+        """
+        self.fieldDict = collections.OrderedDict()
+        if formatData:
+            for fieldData in formatData.get('fields', []):
+                fieldName = fieldData['fieldname']
+                self.addField(fieldName, fieldData)
+        else:
+            formatData = {}
+        self.titleLine = [formatData.get('titleline', '')]
+        self.outputLines = [[line] for line in
+                            formatData.get('outputlines', [])]
+        self.spaceBetween = formatData.get('spacebetween', True)
+        self.formatHtml = formatData.get('formathtml', False)
+        self.useBullets = formatData.get('bullets', False)
+        self.useTables = formatData.get('tables', False)
+        self.childType = formatData.get('childtype', '')
+        self.genericType = formatData.get('generic', '')
+        if 'condition' in formatData:
+            self.conditional = conditional.Conditional(formatData['condition'])
+        if 'childTypeLimit' in formatData:
+            self.childTypeLimit = set(formatData['childTypeLimit'])
+        self.iconName = formatData.get('icon', '')
+        self.outputSeparator = formatData.get('outputsep',
+                                              _defaultOutputSeparator)
+        for key in formatData.keys():
+            if key.startswith('cond-'):
+                self.savedConditionText[key[5:]] = formatData[key]
+
+    def storeFormat(self):
+        """Return JSON format data for this format.
+        """
+        formatData = {}
+        formatData['formatname'] = self.name
+        formatData['fields'] = [field.formatData() for field in self.fields()]
+        formatData['titleline'] = self.getTitleLine()
+        formatData['outputlines'] = self.getOutputLines()
+        if not self.spaceBetween:
+            formatData['spacebetween'] = False
+        if self.formatHtml:
+            formatData['formathtml'] = True
+        if self.useBullets:
+            formatData['bullets'] = True
+        if self.useTables:
+            formatData['tables'] = True
+        if self.childType:
+            formatData['childtype'] = self.childType
+        if self.genericType:
+            formatData['generic'] = self.genericType
+        if self.conditional:
+            formatData['condition'] = self.conditional.conditionStr()
+        if self.childTypeLimit:
+            formatData['childTypeLimit'] = sorted(list(self.childTypeLimit))
+        if self.iconName:
+            formatData['icon'] = self.iconName
+        if self.outputSeparator != _defaultOutputSeparator:
+            formatData['outputsep'] = self.outputSeparator
+        for key, text in self.savedConditionText.items():
+            formatData['cond-' + key] = text
+        return formatData
+
+    def copySettings(self, sourceFormat):
+        """Copy all settings from another format to this one.
+
+        Arguments:
+            sourceFormat -- the format to copy
+        """
+        self.name = sourceFormat.name
+        self.readFormat(sourceFormat.storeFormat())
+        self.siblingPrefix = sourceFormat.siblingPrefix
+        self.siblingSuffix = sourceFormat.siblingSuffix
+        self.outputLines = sourceFormat.getOutputLines(False)
+        self.origOutputLines = sourceFormat.getOutputLines()
+        self.updateLineParsing()
+
     def fields(self):
         """Return list of all fields.
         """
@@ -103,27 +159,30 @@ class NodeFormat:
         """
         return list(self.fieldDict.keys())
 
-    def formatTitle(self, node):
+    def formatTitle(self, node, spotRef=None):
         """Return a string with formatted title data.
 
         Arguments:
             node -- the node used to get data for fields
+            spotRef -- optional, used for ancestor field refs
         """
         line = ''.join([part.outputText(node, True, self.formatHtml)
                         if hasattr(part, 'outputText') else part
-                        for part in self.lineList[0]])
+                        for part in self.titleLine])
         return line.strip().split('\n', 1)[0]   # truncate to 1st line
 
-    def formatOutput(self, node, plainText=False, keepBlanks=False):
+    def formatOutput(self, node, plainText=False, keepBlanks=False,
+                     spotRef=None):
         """Return a list of formatted text output lines.
 
         Arguments:
             node -- the node used to get data for fields
             plainText -- if True, remove HTML markup from fields and formats
             keepBlanks -- if True, keep lines with empty fields
+            spotRef -- optional, used for ancestor field refs
         """
         result = []
-        for lineData in self.lineList[1:]:
+        for lineData in self.outputLines:
             line = ''
             numEmptyFields = 0
             numFullFields = 0
@@ -150,33 +209,29 @@ class NodeFormat:
                     result[-1] += endTagMatch.group(1)
         return result
 
-    def addField(self, name, attrs=None):
+    def addField(self, name, fieldData=None):
         """Add a field type with its format to the field list.
 
         Arguments:
             name -- the field name string
-            attrs -- the attributes that define this field's format
+            fieldData -- the dict that defines this field's format
         """
-        if not attrs:
-            attrs = {}
-        typeName = '{}Field'.format(attrs.get('type', 'Text'))
-        field = getattr(fieldformat, typeName, fieldformat.TextField)(name,
-                                                                      attrs)
+        if not fieldData:
+            fieldData = {}
+        typeName = '{}Field'.format(fieldData.get('fieldtype', 'Text'))
+        fieldClass = getattr(fieldformat, typeName, fieldformat.TextField)
+        field = fieldClass(name, fieldData)
         self.fieldDict[name] = field
-        if attrs.get('idref', '').startswith('y'):
-            self.idField = field
-        elif not self.idField:
-            self.idField = list(self.fieldDict.values())[0]
 
-    def addFieldIfNew(self, name, attrs=None):
+    def addFieldIfNew(self, name, fieldData=None):
         """Add a field type to the field list if not already there.
 
         Arguments:
             name -- the field name string
-            attrs -- the attributes that define this field's format
+            fieldData -- the dict that defines this field's format
         """
         if name not in self.fieldDict:
-            self.addField(name, attrs)
+            self.addField(name, fieldData)
 
     def addFieldList(self, nameList, addFirstTitle=False, addToOutput=False):
         """Add text fields with names given in list.
@@ -185,7 +240,7 @@ class NodeFormat:
         Arguments:
             nameList -- the list of names to add
             addFirstTitle -- if True, use first field for title output format
-            addToOutput -- repelace output lines with all fields if True
+            addToOutput -- replace output lines with all fields if True
         """
         for name in nameList:
             self.addFieldIfNew(name)
@@ -207,25 +262,19 @@ class NodeFormat:
         self.fieldDict = newFieldDict
 
     def removeField(self, field):
-        """Remove all occurances of field from output lines.
+        """Remove all occurances of field from title and output lines.
 
         Arguments:
             field -- the field to be removed
         """
-        for lineData in self.lineList:
+        while field in self.titleLine:
+            self.titleLine.remove(field)
+        for lineData in self.outputLines:
             while field in lineData:
                 lineData.remove(field)
-        self.lineList = [line for line in self.lineList if line]
-        while len(self.lineList) < 2:
-            self.lineList.append([''])
-        self.conditional.removeField(field.name)
-        savedConditions = {}
-        for name, text in self.savedConditionText.items():
-            condition = conditional.Conditional(text, self.name)
-            condition.removeField(field.name)
-            if condition:
-                savedConditions[name] = condition.conditionStr()
-        self.savedConditionText = savedConditions
+        self.outputLines = [line for line in self.outputLines if line]
+        # if len(self.lineList) == 0:
+            # self.lineList.append([''])
 
     def setInitDefaultData(self, data, overwrite=False):
         """Add initial default data from fields into supplied data dict.
@@ -234,7 +283,7 @@ class NodeFormat:
             data -- the data dict to modify
             overwrite -- if true, replace previous data entries
         """
-        for field in self.fieldDict.values():
+        for field in self.fields():
             text = field.getInitDefault()
             if text and (overwrite or not data.get(field.name, '')):
                 data[field.name] = text
@@ -245,10 +294,12 @@ class NodeFormat:
         Converts lines back to whole lines with embedded field names,
         then parse back to individual fields and text.
         """
-        self.lineList = [self.parseLine(line) for line in self.getLines(False)]
-        if self.origLineList:
-            self.origLineList = [self.parseLine(line) for line in
-                                 self.getLines(True)]
+        self.titleLine = self.parseLine(self.getTitleLine())
+        self.outputLines = [self.parseLine(line) for line in
+                            self.getOutputLines(False)]
+        if self.origOutputLines:
+            self.origOutputLines = [self.parseLine(line) for line in
+                                    self.getOutputLines(True)]
 
     def parseLine(self, text):
         """Parse text format line, return list of field types and text.
@@ -274,7 +325,7 @@ class NodeFormat:
             try:
                 if not modifier:
                     return self.fieldDict[fieldName]
-                elif modifier[0] == '*' and modifier == '*' * len(modifier):
+                elif modifier == '*' * len(modifier):
                     return fieldformat.AncestorLevelField(fieldName,
                                                           len(modifier))
                 elif modifier == '?':
@@ -287,27 +338,28 @@ class NodeFormat:
                         level = int(match.group(1))
                         return fieldformat.DescendantCountField(fieldName,
                                                                 level)
-                    else:
-                        return text
                 elif modifier == '!':
-                    if fieldName == uniqueIdFieldName:
-                        return fieldformat.UniqueIdField(fieldName)
-                    else:      # file info field
-                        return (self.parentFormats.fileInfoFormat.
-                                fieldDict[fieldName])
+                    return (self.parentFormats.fileInfoFormat.
+                            fieldDict[fieldName])
             except KeyError:
                 pass
         return text
 
-    def getLines(self, useOriginal=True):
-        """Return text list of lines with field names embedded.
+    def getTitleLine(self):
+        """Return text of title format with field names embedded.
+        """
+        return ''.join([part.sepName() if hasattr(part, 'sepName') else part
+                        for part in self.titleLine])
+
+    def getOutputLines(self, useOriginal=True):
+        """Return text list of output format lines with field names embedded.
 
         Arguments:
             useOriginal -- use original line list, wothout bullet or table mods
         """
-        lines = self.lineList
-        if useOriginal and self.origLineList:
-            lines = self.origLineList
+        lines = self.outputLines
+        if useOriginal and self.origOutputLines:
+            lines = self.origOutputLines
         lines = [''.join([part.sepName() if hasattr(part, 'sepName') else part
                           for part in line])
                  for line in lines]
@@ -319,10 +371,9 @@ class NodeFormat:
         Arguments:
             text -- the new title format line
         """
-        newLine = self.parseLine(text)
-        if not newLine:
-            newLine = ['']
-        self.lineList[0] = newLine
+        self.titleLine = self.parseLine(text)
+        if not self.titleLine:
+            self.titleLine = ['']
 
     def changeOutputLines(self, lines, keepBlanks=False):
         """Replace the output format lines with given list.
@@ -331,18 +382,16 @@ class NodeFormat:
             lines -- a list of replacement format lines
             keepBlanks -- if False, ignore blank lines
         """
-        self.lineList = self.lineList[:1]
-        if not self.lineList:
-            self.lineList = ['']
+        self.outputLines = []
         for line in lines:
             newLine = self.parseLine(line)
             if keepBlanks or newLine:
-                self.lineList.append(newLine)
+                self.outputLines.append(newLine)
         if self.useBullets:
-            self.origLineList = self.lineList[:]
+            self.origOutputLines = self.outputLines[:]
             self.addBullets()
         if self.useTables:
-            self.origLineList = self.lineList[:]
+            self.origOutputLines = self.outputLines[:]
             self.addTables()
 
     def addOutputLine(self, line):
@@ -351,39 +400,9 @@ class NodeFormat:
         Arguments:
             line -- the text line to add
         """
-        if not self.lineList:
-            self.lineList = ['']
         newLine = self.parseLine(line)
         if newLine:
-            self.lineList.append(newLine)
-
-    def xmlAttr(self):
-        """Return a dictionary of this type's attributes.
-        """
-        attrs = {}
-        for i, line in enumerate(self.getLines()):
-            attrs['line{}'.format(i)] = line
-        if not self.spaceBetween:
-            attrs['spacebetween'] = 'n'
-        if self.formatHtml:
-            attrs['formathtml'] = 'y'
-        if self.useBullets:
-            attrs['bullets'] = 'y'
-        if self.useTables:
-            attrs['tables'] = 'y'
-        if self.childType:
-            attrs['childtype'] = self.childType
-        if self.genericType:
-            attrs['generic'] = self.genericType
-        if self.conditional:
-            attrs['condition'] = self.conditional.conditionStr()
-        if self.iconName:
-            attrs['icon'] = self.iconName
-        if self.outputSeparator != _defaultOutputSeparator:
-            attrs['outputsep'] = self.outputSeparator
-        for key, text in self.savedConditionText.items():
-            attrs['cond-' + key] = text
-        return attrs
+            self.outputLines.append(newLine)
 
     def extractTitleData(self, titleString, data):
         """Modifies the data dictionary based on a title string.
@@ -396,7 +415,7 @@ class NodeFormat:
         fields = []
         pattern = ''
         extraText = ''
-        for seg in self.lineList[0]:
+        for seg in self.titleLine:
             if hasattr(seg, 'name'):  # a field segment
                 fields.append(seg)
                 pattern += '(.*)'
@@ -404,21 +423,29 @@ class NodeFormat:
                 pattern += re.escape(seg)
                 extraText += seg
         match = re.match(pattern, titleString)
-        if match:
-            for num, field in enumerate(fields):
-                text = match.group(num + 1)
-                if field.useRichText:
-                    text = xml.sax.saxutils.escape(text)
-                data[field.name] = text
-        elif not extraText.strip():
-            # assign to 1st field if sep is only spaces
-            data[fields[0].name] = titleString
-            for field in fields[1:]:
-                data[field.name] = ''
-        else:
+        try:
+            if match:
+                for num, field in enumerate(fields):
+                    text = match.group(num + 1)
+                    data[field.name] = field.storedTextFromTitle(text)
+            elif not extraText.strip():
+                # assign to 1st field if sep is only spaces
+                text = fields[0].storedTextFromTitle(titleString)
+                data[fields[0].name] = text
+                for field in fields[1:]:
+                    data[field.name] = ''
+            else:
+                return False
+        except ValueError:
             return False
         return True
 
+    def updateDerivedTypes(self):
+        """Update derived types after changes to this generic type.
+        """
+        for derivedType in self.derivedTypes:
+            derivedType.updateFromGeneric(self)
+
     def updateFromGeneric(self, genericType=None, formatsRef=None):
         """Update fields and field types to match a generic type.
 
@@ -440,7 +467,6 @@ class NodeFormat:
             else:
                 newFields[field.name] = copy.deepcopy(field)
         self.fieldDict = newFields
-        self.idField = self.fieldDict[genericType.idField.name]
         self.updateLineParsing()
 
     def addBullets(self):
@@ -448,19 +474,18 @@ class NodeFormat:
         """
         self.siblingPrefix = '<ul>'
         self.siblingSuffix = '</ul>'
-        lines = self.getLines()
-        if len(lines) > 1:
-            lines[1] = '<li>' + lines[1]
+        lines = self.getOutputLines()
+        if lines != ['']:
+            lines[0] = '<li>' + lines[0]
             lines[-1] += '</li>'
-        self.origLineList = self.lineList[:]
-        self.lineList = lines
+        self.origOutputLines = self.outputLines[:]
+        self.outputLines = lines
         self.updateLineParsing()
 
     def addTables(self):
         """Add table HTML tags to sibling prefix, suffix and output lines.
         """
-        origLines = self.getLines()
-        lines = [line for line in origLines[1:] if line]
+        lines = [line for line in self.getOutputLines() if line]
         newLines = []
         headings = []
         for line in lines:
@@ -481,9 +506,8 @@ class NodeFormat:
         newLines = ['<td>{0}</td>'.format(line) for line in newLines]
         newLines[0] = '<tr>' + newLines[0]
         newLines[-1] += '</tr>'
-        self.origLineList = self.lineList[:]
-        self.lineList = newLines
-        self.lineList.insert(0, origLines[0])
+        self.origOutputLines = self.outputLines[:]
+        self.outputLines = newLines
         self.updateLineParsing()
 
     def clearBulletsAndTables(self):
@@ -491,32 +515,10 @@ class NodeFormat:
         """
         self.siblingPrefix = ''
         self.siblingSuffix = ''
-        if self.origLineList:
-            self.lineList = self.origLineList
+        if self.origOutputLines:
+            self.outputLines = self.origOutputLines
             self.updateLineParsing()
-        self.origLineList = []
-
-    def updateDerivedTypes(self):
-        """Update derived types after changes to this generic type.
-        """
-        for derivedType in self.derivedTypes:
-            derivedType.updateFromGeneric(self)
-
-    def findLinkField(self):
-        """Return the field most likely to contain a bookmark URL.
-
-        Return None if there are no matches.
-        """
-        availFields = [field for field in self.fieldDict.values() if
-                       field.typeName == 'ExternalLink']
-        if not availFields:
-            return None
-        bestFields = [field for field in availFields if
-                      field.name.lower() ==
-                      imports.bookmarkLinkFieldName.lower()]
-        if bestFields:
-            return bestFields[0]
-        return availFields[0]
+        self.origOutputLines = []
 
     def numberingFieldList(self):
         """Return a list of numbering field names.
@@ -552,17 +554,14 @@ class FileInfoFormat(NodeFormat):
     numPagesFieldName = 'Number_of_Pages'
     def __init__(self, parentFormats):
         """Create a file info format.
-
-        Arguments:
-            parentFormats -- a ref to TreeFormats class
         """
         super().__init__(FileInfoFormat.typeName, parentFormats)
         self.fieldFormatModified = False
         self.addField(FileInfoFormat.fileFieldName)
         self.addField(FileInfoFormat.pathFieldName)
-        self.addField(FileInfoFormat.sizeFieldName, {'type': 'Number'})
-        self.addField(FileInfoFormat.dateFieldName, {'type': 'Date'})
-        self.addField(FileInfoFormat.timeFieldName, {'type': 'Time'})
+        self.addField(FileInfoFormat.sizeFieldName, {'fieldtype': 'Number'})
+        self.addField(FileInfoFormat.dateFieldName, {'fieldtype': 'Date'})
+        self.addField(FileInfoFormat.timeFieldName, {'fieldtype': 'Time'})
         if not sys.platform.startswith('win'):
             self.addField(FileInfoFormat.ownerFieldName)
         # page info only for print header:
@@ -573,39 +572,34 @@ class FileInfoFormat(NodeFormat):
         for field in self.fields():
             field.useFileInfo = True
 
-    def updateFileInfo(self, fileName, fileInfoNode):
+    def updateFileInfo(self, fileObj, fileInfoNode):
         """Update data of file info node.
 
         Arguments:
-            fileName -- the TreeLine file path
+            fileObj -- the TreeLine file path object
             fileInfoNode -- the node to update
         """
         try:
-            status = os.stat(fileName)
-        except OSError:
+            status = fileObj.stat()
+        except (AttributeError, OSError):
             fileInfoNode.data = {}
             return
-        fileInfoNode.data[FileInfoFormat.fileFieldName] = (os.path.
-                                                           basename(fileName))
-        fileInfoNode.data[FileInfoFormat.pathFieldName] = (os.path.
-                                                           dirname(fileName))
-        fileInfoNode.data[FileInfoFormat.sizeFieldName] = str(status[stat.
-                                                                     ST_SIZE])
-        modDateTime = QtCore.QDateTime()
-        modDateTime.setTime_t(status[stat.ST_MTIME])
-        modDateTime = modDateTime.toLocalTime()
-        modDate = modDateTime.date().toString(QtCore.Qt.ISODate)
-        modTime = modDateTime.time().toString()
+        fileInfoNode.data[FileInfoFormat.fileFieldName] = fileObj.name
+        fileInfoNode.data[FileInfoFormat.pathFieldName] = fileObj.parent
+        fileInfoNode.data[FileInfoFormat.sizeFieldName] = str(status.st_size)
+        modDateTime = datetime.datetime.fromtimestamp(status.st_mtime)
+        modDate = modDateTime.date().strftime(fieldformat.DateField.isoFormat)
+        modTime = modDateTime.time().strftime(fieldformat.TimeField.isoFormat)
         fileInfoNode.data[FileInfoFormat.dateFieldName] = modDate
         fileInfoNode.data[FileInfoFormat.timeFieldName] = modTime
         if not sys.platform.startswith('win'):
             try:
-                owner = pwd.getpwuid(status[stat.ST_UID])[0]
+                owner = pwd.getpwuid(status.st_uid)[0]
             except KeyError:
-                owner = repr(status[stat.ST_UID])
+                owner = repr(status.st_uid)
             fileInfoNode.data[FileInfoFormat.ownerFieldName] = owner
 
-    def duplicateFieldFormats(self, altFileFormat):
+    def duplicateFileInfo(self, altFileFormat):
         """Copy field format settings from alternate file format.
 
         Arguments:
