@@ -4,7 +4,7 @@
 # optiondefaults.py, defines defaults for config options
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2018, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -29,21 +29,30 @@ def setGenOptionDefaults(generalOptions):
     BoolOptionItem(generalOptions, 'AutoFileOpen', False,
                    _('Startup Condition'),
                    _('Automatically open last file used'))
-    BoolOptionItem(generalOptions, 'ShowChildPane', True,
+    BoolOptionItem(generalOptions, 'InitShowBreadcrumb', True,
                    _('Startup Condition'),
-                   _('Show child pane in right hand view'))
-    BoolOptionItem(generalOptions, 'ShowDescendants', False,
+                   _('Show breadcrumb ancestor view'))
+    BoolOptionItem(generalOptions, 'InitShowChildPane', True,
+                   _('Startup Condition'),
+                   _('Show child pane in right hand views'))
+    BoolOptionItem(generalOptions, 'InitShowDescendants', False,
                    _('Startup Condition'),
                    _('Show descendants in output view'))
     BoolOptionItem(generalOptions, 'SaveTreeStates', True,
                    _('Startup Condition'),
                    _('Restore tree view states of recent files'))
+    BoolOptionItem(generalOptions, 'PurgeRecentFiles', True,
+                   _('Startup Condition'),
+                   _('Remove inaccessible recent file entries'))
     BoolOptionItem(generalOptions, 'SaveWindowGeom', True,
                    _('Startup Condition'),
                    _('Restore previous window geometry'))
     BoolOptionItem(generalOptions, 'OpenNewWindow', True,
                    _('Features Available'),
                    _('Open files in new windows'))
+    BoolOptionItem(generalOptions, 'EditorOnHover', True,
+                   _('Features Available'),
+                   _('Activate data editors on mouse hover'))
     BoolOptionItem(generalOptions, 'ClickRename', True,
                    _('Features Available'), _('Click node to rename'))
     BoolOptionItem(generalOptions, 'RenameNewNodes', True,
@@ -52,9 +61,6 @@ def setGenOptionDefaults(generalOptions):
                    _('Tree drag && drop available'))
     BoolOptionItem(generalOptions, 'ShowTreeIcons', True,
                    _('Features Available'), _('Show icons in the tree view'))
-    BoolOptionItem(generalOptions, 'ShowUniqueID', False,
-                   _('Features Available'),
-                   _('Show node unique IDs in the Data Edit view'))
     BoolOptionItem(generalOptions, 'ShowMath', True, _('Features Available'),
                    _('Show math fields in the Data Edit view'))
     BoolOptionItem(generalOptions, 'EditNumbering', False,
@@ -66,9 +72,9 @@ def setGenOptionDefaults(generalOptions):
                   _('Minutes between saves\n(set to 0 to disable)'), 1)
     IntOptionItem(generalOptions, 'RecentFiles', 4, 0, 99, _('Recent Files'),
                   _('Number of recent files \nin the file menu'), 1)
-    StringOptionItem(generalOptions, 'EditTimeFormat', 'H:mm:ss', False,
-                     _('Data Editor Formats'), _('Times'), 1)
-    StringOptionItem(generalOptions, 'EditDateFormat', 'MM/dd/yy', False,
+    StringOptionItem(generalOptions, 'EditTimeFormat', '%-H:%M:%S', False,
+                     True, _('Data Editor Formats'), _('Times'), 1)
+    StringOptionItem(generalOptions, 'EditDateFormat', '%m/%d/%y', False, True,
                      _('Data Editor Formats'), _('Dates'), 1)
     ListOptionItem(generalOptions, 'WeekStart', daysOfWeek[-1], daysOfWeek,
                    _('Data Editor Formats'), _('First day\nof week'), 1)
@@ -79,65 +85,51 @@ def setMiscOptionDefaults(miscOptions):
     """Set defaults for miscellaneous config options.
     """
     StringOptionItem = options.StringOptionItem
-    StringOptionItem(miscOptions, 'PrintUnits', 'in', False,
-                     _('Printer Settings'))
-    StringOptionItem(miscOptions, 'SpellCheckPath', '', True, _('Spell Check'))
-    StringOptionItem(miscOptions, 'TreeFont', '', True, _('Fonts'))
-    StringOptionItem(miscOptions, 'OutputFont', '', True, _('Fonts'))
-    StringOptionItem(miscOptions, 'EditorFont', '', True, _('Fonts'))
+    StringOptionItem(miscOptions, 'PrintUnits', 'in', False, True)
+    StringOptionItem(miscOptions, 'SpellCheckPath', '')
+    StringOptionItem(miscOptions, 'AppFont', '', True, True)
+    StringOptionItem(miscOptions, 'TreeFont', '', True, True)
+    StringOptionItem(miscOptions, 'OutputFont', '', True, True)
+    StringOptionItem(miscOptions, 'EditorFont', '', True, True)
 
 def setHistOptionDefaults(historyOptions):
     """Set defaults for history config options.
     """
-    StringOptionItem = options.StringOptionItem
     IntOptionItem = options.IntOptionItem
-    IntOptionItem(historyOptions, 'WindowXSize', 640, 10, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'WindowYSize', 640, 10, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'WindowXPos', -1000, -1000, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'WindowYPos', -1000, -1000, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'TreeSplitPercent', 40, 1, 99,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'OutputSplitPercent', 20, 1, 99,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'EditorSplitPercent', 25, 1, 99,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'TitleSplitPercent', 10, 1, 99,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'ActiveRightView', 0, 0, 2,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'PrintPrevXSize', 0, 0, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'PrintPrevYSize', 0, 0, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'PrintPrevXPos', -1000, -1000, 10000,
-                  _('Window Geometry'))
-    IntOptionItem(historyOptions, 'PrintPrevYPos', -1000, -1000, 10000,
-                  _('Window Geometry'))
+    DataListOptionItem = options.DataListOptionItem
+    IntOptionItem(historyOptions, 'WindowXSize', 640, 10, 10000)
+    IntOptionItem(historyOptions, 'WindowYSize', 640, 10, 10000)
+    IntOptionItem(historyOptions, 'WindowXPos', -1000, -1000, 10000)
+    IntOptionItem(historyOptions, 'WindowYPos', -1000, -1000, 10000)
+    IntOptionItem(historyOptions, 'CrumbSplitPercent', 10, 1, 99)
+    IntOptionItem(historyOptions, 'TreeSplitPercent', 40, 1, 99)
+    IntOptionItem(historyOptions, 'OutputSplitPercent', 20, 1, 99)
+    IntOptionItem(historyOptions, 'EditorSplitPercent', 25, 1, 99)
+    IntOptionItem(historyOptions, 'TitleSplitPercent', 10, 1, 99)
+    IntOptionItem(historyOptions, 'ActiveRightView', 0, 0, 2)
+    IntOptionItem(historyOptions, 'PrintPrevXSize', 0, 0, 10000)
+    IntOptionItem(historyOptions, 'PrintPrevYSize', 0, 0, 10000)
+    IntOptionItem(historyOptions, 'PrintPrevXPos', -1000, -1000, 10000)
+    IntOptionItem(historyOptions, 'PrintPrevYPos', -1000, -1000, 10000)
+    DataListOptionItem(historyOptions, 'RecentFiles', [])
 
 def setToolbarOptionDefaults(toolbarOptions):
     """Set defaults for toolbar geometry and buttons.
     """
     StringOptionItem = options.StringOptionItem
+    DataListOptionItem = options.DataListOptionItem
     IntOptionItem = options.IntOptionItem
-    IntOptionItem(toolbarOptions, 'ToolbarQuantity', 2, 0, 20,
-                  _('Toolbar Geometry'))
-    IntOptionItem(toolbarOptions, 'ToolbarSize', 16, 1, 128,
-                  _('Toolbar Geometry'))
-    StringOptionItem(toolbarOptions, 'ToolbarPosition', '', True,
-                     _('Toolbar Geometry'))
-    StringOptionItem(toolbarOptions, 'Toolbar0',
-                     'FileNew,FileOpen,FileSave,,FilePrintPreview,FilePrint,,'
-                     'EditUndo,EditRedo,,EditCut,EditCopy,EditPaste,,'
-                     'DataConfigType', True, _('Toolbar Commands'))
-    StringOptionItem(toolbarOptions, 'Toolbar1',
-                     'NodeInsertAfter,NodeAddChild,,NodeDelete,NodeIndent,'
-                     'NodeUnindent,,NodeMoveUp,NodeMoveDown,,'
-                     'ViewPrevSelect,ViewNextSelect,,ViewShowDescend',
-                     True, _('Toolbar Commands'))
+    IntOptionItem(toolbarOptions, 'ToolbarQuantity', 2, 0, 20)
+    IntOptionItem(toolbarOptions, 'ToolbarSize', 16, 1, 128)
+    StringOptionItem(toolbarOptions, 'ToolbarPosition', '')
+    DataListOptionItem(toolbarOptions, 'ToolbarCommands',
+                       ['FileNew,FileOpen,FileSave,,FilePrintPreview,'
+                        'FilePrint,,EditUndo,EditRedo,,EditCut,EditCopy,'
+                        'EditPaste,,DataConfigType,ToolsFindText',
+                        'NodeInsertAfter,NodeAddChild,,NodeDelete,NodeIndent,'
+                        'NodeUnindent,,NodeMoveUp,NodeMoveDown,,'
+                        'ViewExpandBranch,ViewCollapseBranch,,'
+                        'ViewPrevSelect,ViewNextSelect,,ViewShowDescend'])
 
 def setKeyboardOptionDefaults(keyboardOptions):
     """Set defaults for keyboard shortcuts.
@@ -162,19 +154,16 @@ def setKeyboardOptionDefaults(keyboardOptions):
     KeyOptionItem(keyboardOptions, 'EditCopy', 'Ctrl+C', 'Edit Menu')
     KeyOptionItem(keyboardOptions, 'EditPaste', 'Ctrl+V', 'Edit Menu')
     KeyOptionItem(keyboardOptions, 'EditPastePlain', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditBoldFont', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditItalicFont', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditUnderlineFont', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditFontSize', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditFontColor', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditExtLink', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditIntLink', '', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditSelectAll', 'Ctrl+L', 'Edit Menu')
-    KeyOptionItem(keyboardOptions, 'EditClearFormat', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteChild', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteBefore', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteAfter', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteCloneChild', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteCloneBefore', '', 'Edit Menu')
+    KeyOptionItem(keyboardOptions, 'EditPasteCloneAfter', '', 'Edit Menu')
     KeyOptionItem(keyboardOptions, 'NodeRename', 'Ctrl+R', 'Node Menu')
+    KeyOptionItem(keyboardOptions, 'NodeAddChild', 'Ctrl+A', 'Node Menu')
     KeyOptionItem(keyboardOptions, 'NodeInsertBefore', 'Ctrl+B', 'Node Menu')
     KeyOptionItem(keyboardOptions, 'NodeInsertAfter', 'Ctrl+I', 'Node Menu')
-    KeyOptionItem(keyboardOptions, 'NodeAddChild', 'Ctrl+A', 'Node Menu')
     KeyOptionItem(keyboardOptions, 'NodeDelete', 'Del', 'Node Menu')
     KeyOptionItem(keyboardOptions, 'NodeIndent', 'Ctrl+Shift+Right',
                   'Node Menu')
@@ -190,10 +179,11 @@ def setKeyboardOptionDefaults(keyboardOptions):
     KeyOptionItem(keyboardOptions, 'DataCopyType', '', 'Data Menu')
     KeyOptionItem(keyboardOptions, 'DataSortNodes', '', 'Data Menu')
     KeyOptionItem(keyboardOptions, 'DataNumbering', '', 'Data Menu')
+    KeyOptionItem(keyboardOptions, 'DataCloneMatches', '', 'Data Menu')
+    KeyOptionItem(keyboardOptions, 'DataDetachClones', '', 'Data Menu')
     KeyOptionItem(keyboardOptions, 'DataFlatCategory', '', 'Data Menu')
     KeyOptionItem(keyboardOptions, 'DataAddCategory', '', 'Data Menu')
-    KeyOptionItem(keyboardOptions, 'DataFlatLink', '', 'Data Menu')
-    KeyOptionItem(keyboardOptions, 'DataArrangeLink', '', 'Data Menu')
+    KeyOptionItem(keyboardOptions, 'DataSwapCategory', '', 'Data Menu')
     KeyOptionItem(keyboardOptions, 'ToolsFindText', 'Ctrl+F', 'Tools Menu')
     KeyOptionItem(keyboardOptions, 'ToolsFindCondition', '', 'Tools Menu')
     KeyOptionItem(keyboardOptions, 'ToolsFindReplace', '', 'Tools Menu')
@@ -204,6 +194,15 @@ def setKeyboardOptionDefaults(keyboardOptions):
     KeyOptionItem(keyboardOptions, 'ToolsShortcuts', '', 'Tools Menu')
     KeyOptionItem(keyboardOptions, 'ToolsToolbars', '', 'Tools Menu')
     KeyOptionItem(keyboardOptions, 'ToolsFonts', '', 'Tools Menu')
+    KeyOptionItem(keyboardOptions, 'FormatBoldFont', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatItalicFont', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatUnderlineFont', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatFontSize', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatFontColor', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatExtLink', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatIntLink', '', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatSelectAll', 'Ctrl+L', 'Format Menu')
+    KeyOptionItem(keyboardOptions, 'FormatClearFormat', '', 'Format Menu')
     KeyOptionItem(keyboardOptions, 'ViewExpandBranch', 'Ctrl+Right',
                   'View Menu')
     KeyOptionItem(keyboardOptions, 'ViewCollapseBranch', 'Ctrl+Left',
@@ -218,6 +217,7 @@ def setKeyboardOptionDefaults(keyboardOptions):
                   'View Menu')
     KeyOptionItem(keyboardOptions, 'ViewTitleList', 'Ctrl+Shift+T',
                   'View Menu')
+    KeyOptionItem(keyboardOptions, 'ViewBreadcrumb', '', 'View Menu')
     KeyOptionItem(keyboardOptions, 'ViewShowChildPane', 'Ctrl+Shift+C',
                   'View Menu')
     KeyOptionItem(keyboardOptions, 'ViewShowDescend', 'Ctrl+Shift+D',
@@ -227,4 +227,6 @@ def setKeyboardOptionDefaults(keyboardOptions):
     KeyOptionItem(keyboardOptions, 'HelpBasic', '', 'Help Menu')
     KeyOptionItem(keyboardOptions, 'HelpFull', '', 'Help Menu')
     KeyOptionItem(keyboardOptions, 'HelpAbout', '', 'Help Menu')
-    KeyOptionItem(keyboardOptions, 'HelpPlugin', '', 'Help Menu')
+    KeyOptionItem(keyboardOptions, 'IncremSearchStart', 'Ctrl+/', 'No Menu')
+    KeyOptionItem(keyboardOptions, 'IncremSearchNext', 'F3', 'No Menu')
+    KeyOptionItem(keyboardOptions, 'IncremSearchPrev', 'Shift+F3', 'No Menu')
