@@ -3,7 +3,7 @@
 #******************************************************************************
 # options.py, provides a class to manage config options
 #
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2018, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -13,10 +13,20 @@
 
 from collections import OrderedDict
 import sys
+import re
+import pathlib
 import os.path
-from PyQt4 import QtCore, QtGui
+import json
+from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QKeySequence
+from PyQt5.QtWidgets import (QButtonGroup, QCheckBox, QComboBox, QDialog,
+                             QDoubleSpinBox, QGridLayout, QGroupBox,
+                             QHBoxLayout, QLabel, QLineEdit, QPushButton,
+                             QRadioButton, QSpinBox, QVBoxLayout)
 import miscdialogs
 
+multipleSpaceRegEx = re.compile(r' {2,}')
+
 
 class StringOptionItem:
     """Class to store and control a string-based config option entry.
@@ -24,8 +34,8 @@ class StringOptionItem:
     Stores the name, value, category and description, provides validation,
     config file output and dialog controls.
     """
-    def __init__(self, optionDict, name, value, emptyOK=True, category='',
-                 description='', columnNum=0):
+    def __init__(self, optionDict, name, value, emptyOK=True,
+                 stripSpaces=False, category='', description='', columnNum=0):
         """Set the parameters and initial value and add to optionDict.
 
         Raises a ValueError if initial validation fails.
@@ -34,6 +44,7 @@ class StringOptionItem:
             name -- the string key for the option
             value -- the string value
             emptyOK -- if False, does not allow empty string
+            stripSpaces -- if True, remove leading, trailing & double spaces
             category -- a string for the option group this belongs to
             description -- a string for use in the control dialog
             columnNum -- the column position for this control in the dialog
@@ -43,6 +54,7 @@ class StringOptionItem:
         self.description = description
         self.columnNum = columnNum
         self.emptyOK = emptyOK
+        self.stripSpaces = stripSpaces
         self.dialogControl = None
         self.value = None
         self.setValue(value)
@@ -59,6 +71,8 @@ class StringOptionItem:
             value -- the string value to set
         """
         value = str(value)
+        if self.stripSpaces:
+            value = multipleSpaceRegEx.sub(' ', value.strip())
         if value != self.value and (value or self.emptyOK):
             self.value = value
             return True
@@ -66,36 +80,23 @@ class StringOptionItem:
             raise ValueError
         return False
 
-    def valueString(self):
-        """Return a string representation of the value.
+    def storedValue(self):
+        """Return the value to be stored in the JSON file.
         """
-        return str(self.value)
+        return self.value
 
-    def outputLine(self, padding=20):
-        """Return an output line for writing to a config file.
-
-        Arguments:
-            padding -- the width to reserve for the name key
-        """
-        return '{0:{1}}{2}\n'.format(self.name, padding, self.valueString())
-
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Uses the current group box if the category matches, otherwise
-        starts a new one.  Returns the current group box.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = self.getGroupBox(rowLayout, currentGroupBox)
         gridLayout = groupBox.layout()
         row = gridLayout.rowCount()
-        label = QtGui.QLabel(self.description, groupBox)
+        label = QLabel(self.description, groupBox)
         gridLayout.addWidget(label, row, 0)
-        self.dialogControl = QtGui.QLineEdit(self.value, groupBox)
+        self.dialogControl = QLineEdit(self.value, groupBox)
         gridLayout.addWidget(self.dialogControl, row, 1)
-        return groupBox
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -104,22 +105,6 @@ class StringOptionItem:
         """
         return self.setValue(self.dialogControl.text())
 
-    def getGroupBox(self, rowLayout, currentGroupBox):
-        """Return the group box for use with this option item category.
-
-        Group box is same if category matches, creates one otherwise.
-        Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
-        """
-        if currentGroupBox and currentGroupBox.title() == self.category:
-            return currentGroupBox
-        newGroupBox = QtGui.QGroupBox(self.category,
-                                      rowLayout.parentWidget())
-        rowLayout.addWidget(newGroupBox)
-        QtGui.QGridLayout(newGroupBox)
-        return newGroupBox
-
 
 class IntOptionItem(StringOptionItem):
     """Class to store and control an integer-based config option entry.
@@ -144,8 +129,8 @@ class IntOptionItem(StringOptionItem):
         """
         self.minimum = minimum
         self.maximum = maximum
-        super().__init__(optionDict, name, value, False, category, description,
-                         columnNum)
+        super().__init__(optionDict, name, value, False, False, category,
+                         description, columnNum)
 
     def setValue(self, value):
         """Sets the value and validates, returns True if OK.
@@ -171,28 +156,23 @@ class IntOptionItem(StringOptionItem):
             return True
         return False
 
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Uses the current group box if the category matches, otherwise
-        starts a new one.  Returns the current group box.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = self.getGroupBox(rowLayout, currentGroupBox)
         gridLayout = groupBox.layout()
         row = gridLayout.rowCount()
-        label = QtGui.QLabel(self.description, groupBox)
+        label = QLabel(self.description, groupBox)
         gridLayout.addWidget(label, row, 0)
-        self.dialogControl = QtGui.QSpinBox(groupBox)
+        self.dialogControl = QSpinBox(groupBox)
         self.dialogControl.setValue(self.value)
         if self.minimum != None:
             self.dialogControl.setMinimum(self.minimum)
         if self.maximum != None:
             self.dialogControl.setMaximum(self.maximum)
         gridLayout.addWidget(self.dialogControl, row, 1)
-        return groupBox
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -225,8 +205,8 @@ class FloatOptionItem(StringOptionItem):
         """
         self.minimum = minimum
         self.maximum = maximum
-        super().__init__(optionDict, name, value, False, category, description,
-                         columnNum)
+        super().__init__(optionDict, name, value, False, False, category,
+                         description, columnNum)
 
     def setValue(self, value):
         """Sets the value and validates, returns True if OK.
@@ -252,28 +232,23 @@ class FloatOptionItem(StringOptionItem):
             return True
         return False
 
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Uses the current group box if the category matches, otherwise
-        starts a new one.  Returns the current group box.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = self.getGroupBox(rowLayout, currentGroupBox)
         gridLayout = groupBox.layout()
         row = gridLayout.rowCount()
-        label = QtGui.QLabel(self.description, groupBox)
+        label = QLabel(self.description, groupBox)
         gridLayout.addWidget(label, row, 0)
-        self.dialogControl = QtGui.QDoubleSpinBox(groupBox)
+        self.dialogControl = QDoubleSpinBox(groupBox)
         self.dialogControl.setValue(self.value)
         if self.minimum != None:
             self.dialogControl.setMinimum(self.minimum)
         if self.maximum != None:
             self.dialogControl.setMaximum(self.maximum)
         gridLayout.addWidget(self.dialogControl, row, 1)
-        return groupBox
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -302,8 +277,8 @@ class BoolOptionItem(StringOptionItem):
             description -- a string for use in the control dialog
             columnNum -- the column position for this control in the dialog
         """
-        super().__init__(optionDict, name, value, False, category, description,
-                         columnNum)
+        super().__init__(optionDict, name, value, False, False, category,
+                         description, columnNum)
 
     def setValue(self, value):
         """Sets the value and validates, returns True if OK.
@@ -328,27 +303,17 @@ class BoolOptionItem(StringOptionItem):
             raise ValueError
         return False
 
-    def valueString(self):
-        """Return a string representation of the value.
-        """
-        return 'yes' if self.value else 'no'
-
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Uses the current group box if the category matches, otherwise
-        starts a new one.  Returns the current group box.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = self.getGroupBox(rowLayout, currentGroupBox)
         gridLayout = groupBox.layout()
         row = gridLayout.rowCount()
-        self.dialogControl = QtGui.QCheckBox(self.description, groupBox)
+        self.dialogControl = QCheckBox(self.description, groupBox)
         self.dialogControl.setChecked(self.value)
         gridLayout.addWidget(self.dialogControl, row, 0, 1, 2)
-        return groupBox
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -379,8 +344,8 @@ class ListOptionItem(StringOptionItem):
             columnNum -- the column position for this control in the dialog
         """
         self.choices = choices
-        super().__init__(optionDict, name, value, False, category, description,
-                         columnNum)
+        super().__init__(optionDict, name, value, False, False, category,
+                         description, columnNum)
 
     def setValue(self, value):
         """Sets the value and validates, returns True if OK.
@@ -399,25 +364,20 @@ class ListOptionItem(StringOptionItem):
             raise ValueError
         return False
 
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Uses the current group box if the category matches, otherwise
-        starts a new one.  Returns the current group box.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = self.getGroupBox(rowLayout, currentGroupBox)
         gridLayout = groupBox.layout()
         row = gridLayout.rowCount()
-        label = QtGui.QLabel(self.description, groupBox)
+        label = QLabel(self.description, groupBox)
         gridLayout.addWidget(label, row, 0)
-        self.dialogControl = QtGui.QComboBox(groupBox)
+        self.dialogControl = QComboBox(groupBox)
         self.dialogControl.addItems(self.choices)
         self.dialogControl.setCurrentIndex(self.choices.index(self.value))
         gridLayout.addWidget(self.dialogControl, row, 1)
-        return groupBox
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -447,7 +407,7 @@ class ChoiceOptionItem(StringOptionItem):
             columnNum -- the column position for this control in the dialog
         """
         self.choices = choices
-        super().__init__(optionDict, name, value, False, category, '',
+        super().__init__(optionDict, name, value, False, False, category, '',
                          columnNum)
 
     def setValue(self, value):
@@ -467,26 +427,22 @@ class ChoiceOptionItem(StringOptionItem):
             raise ValueError
         return False
 
-    def addDialogControl(self, rowLayout, currentGroupBox):
+    def addDialogControl(self, groupBox):
         """Add the labels and controls to a dialog box for this option item.
 
-        Always starts a new group box, returns None (group not reused).
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            groupBox -- the current group box
         """
-        groupBox = QtGui.QGroupBox(self.category, rowLayout.parentWidget())
         rowLayout.addWidget(groupBox)
-        QtGui.QGridLayout(groupBox)
+        QGridLayout(groupBox)
         gridLayout = groupBox.layout()
-        self.dialogControl = QtGui.QButtonGroup(groupBox)
+        self.dialogControl = QButtonGroup(groupBox)
         row = 0
         for choice in self.choices:
-            button = QtGui.QRadioButton(choice, groupBox)
+            button = QRadioButton(choice, groupBox)
             self.dialogControl.addButton(button)
             gridLayout.addWidget(button, row, 0, 1, 2)
             row += 1
-        return None
 
     def updateFromDialog(self):
         """Set the value of this item from the dialog contents.
@@ -499,11 +455,10 @@ class ChoiceOptionItem(StringOptionItem):
 class KeyOptionItem(StringOptionItem):
     """Class to store and control a keyboard key based config option entry.
 
-    Stores the name, value, category and description, provides validation,
-    config file output and dialog controls.
+    Stores the name, value and category, provides validation and config file
+    output.
     """
-    def __init__(self, optionDict, name, value, category='', description='',
-                 columnNum=0):
+    def __init__(self, optionDict, name, value, category=''):
         """Set the parameters and initial value and add to optionDict.
 
         Raises a ValueError if initial validation fails.
@@ -512,11 +467,8 @@ class KeyOptionItem(StringOptionItem):
             name -- the string key for the option
             value -- a numeric or string-based value
             category -- a string for the option group this belongs to
-            description -- a string for use in the control dialog
-            columnNum -- the column position for this control in the dialog
         """
-        super().__init__(optionDict, name, value, True, category, description,
-                         columnNum)
+        super().__init__(optionDict, name, value, True, False, category)
 
     def setValue(self, value):
         """Sets the value and validates, returns True if OK.
@@ -525,9 +477,9 @@ class KeyOptionItem(StringOptionItem):
         or if the value is unchanged.
         Raises a ValueError if validation fails without an old value.
         Arguments:
-            value -- a numeric or string-based value to set
+            value -- a key string value to set
         """
-        key = QtGui.QKeySequence(value)
+        key = QKeySequence(value)
         if value and key.isEmpty():
             if self.value == None:
                 raise ValueError
@@ -537,35 +489,49 @@ class KeyOptionItem(StringOptionItem):
             return True
         return False
 
-    def valueString(self):
-        """Return a string representation of the value.
+    def storedValue(self):
+        """Return the value to be stored in the JSON file.
         """
         return self.value.toString()
 
-    def addDialogControl(self, rowLayout, currentGroupBox):
-        """Add the labels and controls to a dialog box for this option item.
 
-        Not implemented yet (needed here?).
-        Returns the current group box.
+class DataListOptionItem(StringOptionItem):
+    """Class to store an arbitrary length list containing various data.
+
+    Stores the name and value, provides validation and config file output.
+    """
+    def __init__(self, optionDict, name, value):
+        """Set the parameters and initial value and add to optionDict.
+
+        Raises a ValueError if initial validation fails.
         Arguments:
-            rowLayout -- the vertical box layout holding the group boxes
-            currentGroupBox -- the currently used group box
+            optionDict -- a dictionary to add this option item to
+            name -- the string key for the option
+            value -- a list containg other basic data types
         """
-        return currentGroupBox
+        super().__init__(optionDict, name, value)
 
-    def updateFromDialog(self):
-        """Set the value of this item from the dialog contents.
+    def setValue(self, value):
+        """Sets the value and validates, returns True if OK.
 
-        Not implemented yet (needed here?).
-        Return True if successfully set.
+        Returns False if validation fails but the old value is OK,
+        or if the value is unchanged.
+        Raises a ValueError if validation fails without an old value.
+        Arguments:
+            value -- a list containg other basic data types
         """
+        if isinstance(value, list) and value != self.value:
+            self.value = value
+            return True
+        if self.value == None:
+            raise ValueError
         return False
 
 
 class Options(OrderedDict):
     """Class to store and control the config options for a program.
     """
-    basePath = ''
+    basePath = None
     def __init__(self, fileName='', progName='', version='', coDirName=''):
         """Initialize and set the path to the config file.
 
@@ -579,68 +545,62 @@ class Options(OrderedDict):
         """
         super().__init__()
         self.modified = False
-        self.path = ''
+        self.path = pathlib.Path()
 
         if not fileName:
             return    # no storage without fileName (temporary options only)
         appDirName = '{0}-{1}'.format(progName.lower(), version)
-        if sys.platform.startswith('win'):    # Windows
-            fileNameSuffix = '.ini'
-            if not Options.basePath:
-                userPath = os.path.join(os.environ.get('APPDATA', ''),
-                                        coDirName, appDirName)
-        else:    # Linux, etc.
-            fileNameSuffix = 'rc'
-            if not Options.basePath:
-                userPath = os.path.join(os.environ.get('HOME', ''),
-                                        '.' + appDirName)
+        fileNameSuffix = '.ini' if sys.platform.startswith('win') else 'rc'
+
         if not Options.basePath:
-            if os.path.exists(userPath):
+            if sys.platform.startswith('win'):    # Windows
+                userPath = (pathlib.Path(os.environ.get('APPDATA', '')) /
+                            coDirName / appDirName)
+            else:    # Linux, etc.
+                userPath = (pathlib.Path(os.path.expanduser('~')) /
+                            ('.' + appDirName))
+            if userPath.is_dir():
                 Options.basePath = userPath
             else:
-                modPath = os.path.dirname(os.path.abspath(sys.path[0]))
-                modConfigPath = os.path.join(modPath, 'config')
-                if os.path.exists(modConfigPath):
+                modPath = pathlib.Path(sys.path[0]).resolve()
+                modConfigPath = modPath / 'config'
+                if modConfigPath.is_dir():
                     Options.basePath = modConfigPath
-                elif os.access(modPath, os.W_OK):
+                elif os.access(str(modPath), os.W_OK):
                     dialog = miscdialogs.RadioChoiceDialog(progName,
                               _('Choose configuration file location'),
                               [(_('User\'s home directory (recommended)'), 0),
                                (_('Program directory (for portable use)'), 1)])
-                    if dialog.exec_() != QtGui.QDialog.Accepted:
+                    if dialog.exec_() != QDialog.Accepted:
                         sys.exit(0)
                     if dialog.selectedButton() == 1:
                         Options.basePath = modConfigPath
                 if not Options.basePath:
                     Options.basePath = userPath
             try:
-                if not os.path.exists(Options.basePath):
-                    os.makedirs(Options.basePath)
-                iconPath = os.path.join(Options.basePath, 'icons')
-                if not os.path.exists(iconPath):
-                    os.makedirs(iconPath)
-                templatePath = os.path.join(Options.basePath, 'templates')
-                if not os.path.exists(templatePath):
-                    os.makedirs(templatePath)
-                pluginPath = os.path.join(Options.basePath, 'plugins')
-                if not os.path.exists(pluginPath):
-                    os.makedirs(pluginPath)
+                if not Options.basePath.is_dir():
+                    Options.basePath.mkdir(parents=True)
+                iconPath = Options.basePath / 'icons'
+                if not iconPath.is_dir():
+                    iconPath.mkdir()
+                templatePath = Options.basePath / 'templates'
+                if not templatePath.is_dir():
+                    templatePath.mkdir()
+                templateExportPath = templatePath / 'exports'
+                if not templateExportPath.is_dir():
+                    templateExportPath.mkdir()
             except OSError:
-                Options.basePath = ''
+                Options.basePath = None
         if Options.basePath:
-            self.path = os.path.join(Options.basePath,
-                                     fileName + fileNameSuffix)
+            self.path = Options.basePath / (fileName + fileNameSuffix)
 
-    def getValue(self, name, defaultValue=False):
-        """Return a value from the given option key.
+    def __getitem__(self, name):
+        """Return the value of an option when called as options[name].
 
         Arguments:
             name -- the string key for the option
-            defaultValue -- if True, return the default value, not current one
         """
-        if not defaultValue:
-            return self[name].value
-        return self[name].defaultValue
+        return self.get(name).value
 
     def getDefaultValue(self, name):
         """Return the initially set default value from the given option key.
@@ -648,7 +608,7 @@ class Options(OrderedDict):
         Arguments:
             name -- the string key for the option
         """
-        return self[name].defaultValue
+        return self.get(name).defaultValue
 
     def changeValue(self, name, value):
         """Set a new value for the given option key.
@@ -658,11 +618,21 @@ class Options(OrderedDict):
             name -- the string key for the option
             value -- a value or a string defining the value
         """
-        if self[name].setValue(value):
+        if self.get(name).setValue(value):
             self.modified = True
             return True
         return False
 
+    def resetToDefaults(self, keyList):
+        """Reset all options with the given keys to original default values.
+
+        Arguments:
+            keyList -- a list of option names to reset
+        """
+        for key in keyList:
+            self.get(key).setValue(self.get(key).defaultValue)
+        self.modified = True
+
     def removeValue(self, name):
         """Remove the value from the option list if possible.
 
@@ -683,25 +653,14 @@ class Options(OrderedDict):
         Only updates existing config items.
         """
         try:
-            with open(self.path, 'r', encoding='utf-8') as f:
-                lastCategory = ''
-                for line in f:
-                    line = line.strip()
-                    if line and not line.startswith('#'):
-                        try:
-                            name, value = line.split(None, 1)
-                        except ValueError:
-                            name = line
-                            value = ''
-                        try:
-                            self[name].setValue(value)
-                            lastCategory = self[name].category
-                        except KeyError:
-                            # create a blank default entry if none exists
-                            StringOptionItem(self, name, '', True,
-                                             lastCategory)
-                            self[name].setValue(value)
-        except IOError:
+            with self.path.open('r', encoding='utf-8') as f:
+                data = json.load(f)
+                for key, value in data.items():
+                    try:
+                        self.get(key).setValue(value)
+                    except AttributeError:
+                        pass
+        except (IOError, ValueError):
             self.writeFile()
 
     def writeFile(self):
@@ -709,45 +668,45 @@ class Options(OrderedDict):
 
         Raises IOError on failure.
         """
-        with open(self.path, 'w', encoding='utf-8') as f:
-            padding = max(len(option.name) for option in self.values()) + 2
-            prevCategory = ''
-            for option in self.values():
-                if option.category and option.category != prevCategory:
-                    f.write('\n# {}:\n'.format(option.category))
-                    prevCategory = option.category
-                f.write(option.outputLine(padding))
+        with self.path.open('w', encoding='utf-8') as f:
+            data = OrderedDict([(key, obj.storedValue()) for (key, obj) in
+                                self.items()])
+            json.dump(data, f, indent=0)
         self.modified = False
 
 
-class OptionDialog(QtGui.QDialog):
+class OptionDialog(QDialog):
     """Provides a dialog with controls for all options in an Options instance.
     """
     def __init__(self, options, parent=None):
         super().__init__(parent)
         self.options = options
-        self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint |
-                            QtCore.Qt.WindowSystemMenuHint)
-        topLayout = QtGui.QVBoxLayout(self)
+        self.setWindowFlags(Qt.Dialog | Qt.WindowTitleHint |
+                            Qt.WindowCloseButtonHint)
+        topLayout = QVBoxLayout(self)
         self.setLayout(topLayout)
-        columnLayout = QtGui.QHBoxLayout()
+        columnLayout = QHBoxLayout()
         topLayout.addLayout(columnLayout)
-        rowLayout = QtGui.QVBoxLayout()
+        rowLayout = QVBoxLayout()
         columnLayout.addLayout(rowLayout)
         groupBox = None
         for option in self.options.values():
             if option.columnNum > columnLayout.count() - 1:
-                rowLayout = QtGui.QVBoxLayout()
+                rowLayout = QVBoxLayout()
                 columnLayout.addLayout(rowLayout)
-            groupBox = option.addDialogControl(rowLayout, groupBox)
+            if not groupBox or groupBox.title() != option.category:
+                groupBox = QGroupBox(option.category)
+                rowLayout.addWidget(groupBox)
+                QGridLayout(groupBox)
+            option.addDialogControl(groupBox)
 
-        ctrlLayout = QtGui.QHBoxLayout()
+        ctrlLayout = QHBoxLayout()
         topLayout.addLayout(ctrlLayout)
         ctrlLayout.addStretch(0)
-        okButton = QtGui.QPushButton(_('&OK'))
+        okButton = QPushButton(_('&OK'))
         ctrlLayout.addWidget(okButton)
         okButton.clicked.connect(self.accept)
-        cancelButton = QtGui.QPushButton(_('&Cancel'))
+        cancelButton = QPushButton(_('&Cancel'))
         ctrlLayout.addWidget(cancelButton)
         cancelButton.clicked.connect(self.reject)
 
