@@ -1,4 +1,4 @@
-# $Id: p3.py,v 1.2 2003/11/18 19:04:03 phr Exp phr $
+#!/usr/bin/env python3
 
 # Simple p3 encryption "algorithm": it's just SHA used as a stream
 # cipher in output feedback mode.  
@@ -12,41 +12,38 @@
 
 # Please include this revision number in any bug reports: $Revision: 1.2 $.
 
-from string import join
+# Modified by Doug Bell to be compatible with Python 3
+
 from array import array
 from time import time
 import struct
-try:
-    import hashlib
-    shaHash = hashlib.sha1
-except ImportError:
-    import sha         # stay compatible with Python 2.4
-    shaHash = sha.new
+import hashlib
+shaHash = hashlib.sha1
 
 class CryptError(Exception): pass
-def _hash(str): return shaHash(str).digest()
+def _hash(text): return shaHash(text).digest()
 
 _ivlen = 16
 _maclen = 8
-_state = _hash(`time()`)
+_state = _hash(repr(time()).encode())
 
 # added by Doug Bell for compatibility with 64-bit systems
-_arraytype = filter(lambda x: struct.calcsize(x) == 4, 'LIH')[0]
+_arraytype = [t for t in 'LIH' if struct.calcsize(t) == 4][0]
 
 try:
     import os
-    _pid = `os.getpid()`
-except ImportError, AttributeError:
+    _pid = repr(os.getpid()).encode()
+except (ImportError, AttributeError):
     _pid = ''
 
 def _expand_key(key, clen):
-    blocks = (clen+19)/20
+    blocks = (clen+19)//20
     xkey=[]
     seed=key
-    for i in xrange(blocks):
+    for i in range(blocks):
         seed=shaHash(key+seed).digest()
         xkey.append(seed)
-    j = join(xkey,'')
+    j = b''.join(xkey)
     return array(_arraytype, j)
 
 def p3_encrypt(plain,key):
@@ -58,7 +55,7 @@ def p3_encrypt(plain,key):
     # the state.  This way if two threads compute a nonce from the
     # same data, they won't both get the same nonce.  (There's still
     # a small danger of a duplicate nonce--see below).
-    _state = 'X'+_state
+    _state = b'X'+_state
 
     # Attempt to make nlist unique for each call, so we can get a
     # unique nonce.  It might be good to include a process ID or
@@ -68,15 +65,16 @@ def p3_encrypt(plain,key):
     # two separate Python instances at the same time), you might get
     # identical ciphertexts for the identical plaintexts, which would
     # be a security failure in some applications.  Be careful.
-    nlist = [`time()`, _pid, _state, `len(plain)`,plain, key]
-    nonce = H(join(nlist,','))[:_ivlen]
-    _state = H('update2'+_state+nonce)
-    k_enc, k_auth = H('enc'+key+nonce), H('auth'+key+nonce)
+    nlist = [repr(time()).encode(), _pid, _state,
+             repr(len(plain)).encode(),plain, key]
+    nonce = H(b','.join(nlist))[:_ivlen]
+    _state = H(b'update2'+_state+nonce)
+    k_enc, k_auth = H(b'enc'+key+nonce), H(b'auth'+key+nonce)
     n=len(plain)                        # cipher size not counting IV
 
-    stream = array(_arraytype, plain+'0000'[n&3:]) # pad to fill 32-bit words
+    stream = array(_arraytype, plain+b'0000'[n&3:]) # pad to fill 32-bit words
     xkey = _expand_key(k_enc, n+4)
-    for i in xrange(len(stream)):
+    for i in range(len(stream)):
         stream[i] = stream[i] ^ xkey[i]
     ct = nonce + stream.tostring()[:n]
     auth = _hmac(ct, k_auth)
@@ -86,17 +84,17 @@ def p3_decrypt(cipher,key):
     H = _hash
     n=len(cipher)-_ivlen-_maclen        # length of ciphertext
     if n < 0:
-        raise CryptError, "invalid ciphertext"
+        raise CryptError("invalid ciphertext")
     nonce,stream,auth = \
-      cipher[:_ivlen], cipher[_ivlen:-_maclen]+'0000'[n&3:],cipher[-_maclen:]
-    k_enc, k_auth = H('enc'+key+nonce), H('auth'+key+nonce)
+      cipher[:_ivlen], cipher[_ivlen:-_maclen]+b'0000'[n&3:],cipher[-_maclen:]
+    k_enc, k_auth = H(b'enc'+key+nonce), H(b'auth'+key+nonce)
     vauth = _hmac (cipher[:-_maclen], k_auth)[:_maclen]
     if auth != vauth:
-        raise CryptError, "invalid key or ciphertext"
+        raise CryptError("invalid key or ciphertext")
 
     stream = array(_arraytype, stream)
     xkey = _expand_key (k_enc, n+4)
-    for i in xrange (len(stream)):
+    for i in range (len(stream)):
         stream[i] = stream[i] ^ xkey[i]
     plain = stream.tostring()[:n]
     return plain
@@ -104,25 +102,24 @@ def p3_decrypt(cipher,key):
 # RFC 2104 HMAC message authentication code
 # This implementation is faster than Python 2.2's hmac.py, and also works in
 # old Python versions (at least as old as 1.5.2).
-from string import translate
 def _hmac_setup():
     global _ipad, _opad, _itrans, _otrans
     _itrans = array('B',[0]*256)
     _otrans = array('B',[0]*256)    
-    for i in xrange(256):
+    for i in range(256):
         _itrans[i] = i ^ 0x36
         _otrans[i] = i ^ 0x5c
     _itrans = _itrans.tostring()
     _otrans = _otrans.tostring()
 
-    _ipad = '\x36'*64
-    _opad = '\x5c'*64
+    _ipad = b'\x36'*64
+    _opad = b'\x5c'*64
 
 def _hmac(msg, key):
     if len(key)>64:
         key=shaHash(key).digest()
-    ki = (translate(key,_itrans)+_ipad)[:64] # inner
-    ko = (translate(key,_otrans)+_opad)[:64] # outer
+    ki = (key.translate(_itrans)+_ipad)[:64] # inner
+    ko = (key.translate(_otrans)+_opad)[:64] # outer
     return shaHash(ko+shaHash(ki+msg).digest()).digest()
 
 #
@@ -132,10 +129,10 @@ def _hmac(msg, key):
 def _time_p3(n=1000,len=20):
     plain="a"*len
     t=time()
-    for i in xrange(n):
-        p3_encrypt(plain,"abcdefgh")
+    for i in range(n):
+        p3_encrypt(plain,b"abcdefgh")
     dt=time()-t
-    print "plain p3:", n,len,dt,"sec =",n*len/dt,"bytes/sec"
+    print("plain p3:", n,len,dt,"sec =",n*len//dt,"bytes/sec")
 
 def _speed():
     _time_p3(len=5)
@@ -147,24 +144,24 @@ def _test():
     e=p3_encrypt
     d=p3_decrypt
 
-    plain="test plaintext"
-    key = "test key"
+    plain=b"test plaintext"
+    key = b"test key"
     c1 = e(plain,key)
     c2 = e(plain,key)
     assert c1!=c2
     assert d(c2,key)==plain
     assert d(c1,key)==plain
-    c3 = c2[:20]+chr(1+ord(c2[20]))+c2[21:] # change one ciphertext character
+    c3 = c2[:20]+chr(1+c2[20]).encode()+c2[21:] # change one ciphertext char
 
     try:
-        print d(c3,key)         # should throw exception
-        print "auth verification failure"
+        print(d(c3,key))         # should throw exception
+        print("auth verification failure")
     except CryptError:
         pass
 
     try:
-        print d(c2,'wrong key')         # should throw exception
-        print "test failure"
+        print(d(c2,b'wrong key'))        # should throw exception
+        print("test failure")
     except CryptError:
         pass
 
