@@ -4,7 +4,7 @@
 # printdata.py, provides a class for printing
 #
 # TreeLine, an information storage program
-# Copyright (C) 2015, Douglas W. Bell
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -13,12 +13,20 @@
 #******************************************************************************
 
 import os.path
-from PyQt4 import QtCore, QtGui
+import enum
+from PyQt5.QtCore import QMarginsF, QSizeF, Qt
+from PyQt5.QtGui import (QAbstractTextDocumentLayout, QFontMetrics,
+                         QPageLayout, QPageSize, QPainter, QTextDocument)
+from PyQt5.QtWidgets import QApplication, QDialog, QFileDialog, QMessageBox
+from PyQt5.QtPrintSupport import QPrintDialog, QPrinter
 import treeoutput
 import printdialogs
 import globalref
 
-entireTree, selectBranch, selectNode = range(3)
+PrintScope = enum.IntEnum('PrintScope', 'entireTree selectBranch selectNode')
+_defaultMargin = 0.5
+_defaultHeaderPos = 0.2
+_defaultColumnSpace = 0.5
 
 
 class PrintData:
@@ -34,10 +42,11 @@ class PrintData:
         """
         self.localControl = localControl
         self.outputGroup = None
-        self.printWhat = entireTree
+        self.printWhat = PrintScope.entireTree
         self.includeRoot = True
         self.openOnly = False
-        self.printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
+        self.printer = QPrinter(QPrinter.HighResolution)
+        self.pageLayout = self.printer.pageLayout()
         self.setDefaults()
         self.adjustSpacing()
 
@@ -47,13 +56,14 @@ class PrintData:
         self.drawLines = True
         self.widowControl = True
         self.indentFactor = 2.0
-        self.printer.setPaperSize(QtGui.QPrinter.Letter)
-        self.printer.setOrientation(QtGui.QPrinter.Portrait)
-        self.printer.setPageMargins(0.5, 0.5, 0.5, 0.5, QtGui.QPrinter.Inch)
-        self.headerMargin = 0.2
-        self.footerMargin = 0.2
+        self.pageLayout.setUnits(QPageLayout.Inch)
+        self.pageLayout.setPageSize(QPageSize(QPageSize.Letter))
+        self.pageLayout.setOrientation(QPageLayout.Portrait)
+        self.pageLayout.setMargins(QMarginsF(*(_defaultMargin,) * 4))
+        self.headerMargin = _defaultHeaderPos
+        self.footerMargin = _defaultHeaderPos
         self.numColumns = 1
-        self.columnSpacing = 0.5
+        self.columnSpacing = _defaultColumnSpace
         self.headerText = ''
         self.footerText = ''
         self.useDefaultFont = True
@@ -62,8 +72,8 @@ class PrintData:
     def setDefaultFont(self):
         """Set the default font initially and based on an output font change.
         """
-        self.defaultFont = QtGui.QTextDocument().defaultFont()
-        fontName = globalref.miscOptions.getValue('OutputFont')
+        self.defaultFont = QTextDocument().defaultFont()
+        fontName = globalref.miscOptions['OutputFont']
         if fontName:
             self.defaultFont.fromString(fontName)
         if self.useDefaultFont:
@@ -72,119 +82,135 @@ class PrintData:
     def adjustSpacing(self):
         """Adjust line spacing & indent size based on font & indent factor.
         """
-        self.lineSpacing = QtGui.QFontMetrics(self.mainFont,
-                                              self.printer).lineSpacing()
+        self.lineSpacing = QFontMetrics(self.mainFont,
+                                        self.printer).lineSpacing()
         self.indentSize = self.indentFactor * self.lineSpacing
 
-    def xmlAttr(self):
+    def fileData(self):
         """Return a dictionary of non-default settings for storage.
         """
-        attrs = {}
+        data = {}
         if not self.drawLines:
-            attrs['printlines'] = 'n'
+            data['printlines'] = False
         if not self.widowControl:
-            attrs['printwidowcontrol'] = 'n'
+            data['printwidowcontrol'] = False
         if self.indentFactor != 2.0:
-            attrs['printindentfactor'] = repr(self.indentFactor)
-        if self.printer.paperSize() == QtGui.QPrinter.Custom:
+            data['printindentfactor'] = self.indentFactor
+        pageSizeId = self.pageLayout.pageSize().id()
+        if pageSizeId == QPageSize.Custom:
             paperWidth, paperHeight = self.roundedPaperSize()
-            attrs['printpaperwidth'] = repr(paperWidth)
-            attrs['printpaperheight'] = repr(paperHeight)
-        elif self.printer.paperSize() != QtGui.QPrinter.Letter:
-            paperSizeDict = dict((num, attrib) for attrib, num in
-                                 vars(QtGui.QPrinter).items()
-                                 if isinstance(num, QtGui.QPrinter.PageSize))
-            attrs['printpapersize'] = paperSizeDict[self.printer.paperSize()]
-        if self.printer.orientation() != QtGui.QPrinter.Portrait:
-            attrs['printportrait'] = 'n'
-        if self.roundedMargins() != [0.5] * 4:
-            attrs['printmargins'] = ' '.join([repr(margin) for margin in
-                                              self.roundedMargins()])
-        if self.headerMargin != 0.2:
-            attrs['printheadermargin'] = repr(self.headerMargin)
-        if self.footerMargin != 0.2:
-            attrs['printfootermargin'] = repr(self.footerMargin)
+            data['printpaperwidth'] = paperWidth
+            data['printpaperheight'] = paperHeight
+        elif pageSizeId != QPageSize.Letter:
+            data['printpapersize'] = self.paperSizeName(pageSizeId)
+        if self.pageLayout.orientation() != QPageLayout.Portrait:
+            data['printportrait'] = False
+        if self.roundedMargins() != (_defaultMargin,) * 4:
+            data['printmargins'] = list(self.roundedMargins())
+        if self.headerMargin != _defaultHeaderPos:
+            data['printheadermargin'] = self.headerMargin
+        if self.footerMargin != _defaultHeaderPos:
+            data['printfootermargin'] = self.footerMargin
         if self.numColumns > 1:
-            attrs['printnumcolumns'] = repr(self.numColumns)
-        if self.columnSpacing != 0.5:
-            attrs['printcolumnspace'] = repr(self.columnSpacing)
+            data['printnumcolumns'] = self.numColumns
+        if self.columnSpacing != _defaultColumnSpace:
+            data['printcolumnspace'] = self.columnSpacing
         if self.headerText:
-            attrs['printheadertext'] = self.headerText
+            data['printheadertext'] = self.headerText
         if self.footerText:
-            attrs['printfootertext'] = self.footerText
+            data['printfootertext'] = self.footerText
         if not self.useDefaultFont:
-            attrs['printfont'] = self.mainFont.toString()
-        return attrs
+            data['printfont'] = self.mainFont.toString()
+        return data
 
-    def restoreXmlAttrs(self, attrs):
+    def readData(self, data):
         """Restore saved settings from a dictionary.
 
         Arguments:
-            attrs -- a dictionary of stored non-default settings
+            data -- a dictionary of stored non-default settings
         """
         self.setDefaults()   # necessary for undo/redo
-        if attrs.get('printlines', '').startswith('n'):
-            self.drawLines = False
-        if attrs.get('printwidowcontrol', '').startswith('n'):
-            self.widowControl = False
-        if 'printindentfactor' in attrs:
-            self.indentFactor = float(attrs['printindentfactor'])
-        if 'printpapersize' in attrs:
-            self.printer.setPaperSize(getattr(QtGui.QPrinter,
-                                              attrs['printpapersize']))
-        if 'printpaperwidth' in attrs and 'printpaperheight' in attrs:
-            width =  float(attrs['printpaperwidth'])
-            height = float(attrs['printpaperheight'])
-            self.printer.setPaperSize(QtCore.QSizeF(width, height),
-                                      QtGui.QPrinter.Inch)
-        if attrs.get('printportrait', '').startswith('n'):
-            self.printer.setOrientation(QtGui.QPrinter.Landscape)
-        if 'printmargins' in attrs:
-            margins = [float(margin) for margin in
-                       attrs['printmargins'].split()]
-            margins.append(QtGui.QPrinter.Inch)
-            self.printer.setPageMargins(*margins)
-        if 'printheadermargin' in attrs:
-            self.headerMargin = float(attrs['printheadermargin'])
-        if 'printfootermargin' in attrs:
-            self.footerMargin = float(attrs['printfootermargin'])
-        if 'printnumcolumns' in attrs:
-            self.numColumns = int(attrs['printnumcolumns'])
-        if 'printcolumnspace' in attrs:
-            self.columnSpacing = float(attrs['printcolumnspace'])
-        self.headerText = attrs.get('printheadertext', '')
-        self.footerText = attrs.get('printfootertext', '')
-        if 'printfont' in attrs:
+        self.drawLines = data.get('printlines', True)
+        self.widowControl = data.get('printwidowcontrol', True)
+        self.indentFactor = data.get('printindentfactor', 2.0)
+        if 'printpapersize' in data:
+            self.pageLayout.setPageSize(QPageSize(getattr(QPageSize,
+                                                  data['printpapersize'])))
+            self.pageLayout.setMargins(QMarginsF(*(_defaultMargin,) * 4))
+        if 'printpaperwidth' in data and 'printpaperheight' in data:
+            width =  data['printpaperwidth']
+            height = data['printpaperheight']
+            self.pageLayout.setPageSize(QPageSize(QSizeF(width, height),
+                                        QPageSize.Inch))
+            self.pageLayout.setMargins(QMarginsF(*(_defaultMargin,) * 4))
+        if not data.get('printportrait', True):
+            self.pageLayout.setOrientation(QPageLayout.Landscape)
+        if 'printmargins' in data:
+            margins = data['printmargins']
+            self.pageLayout.setMargins(QMarginsF(*margins))
+        self.headerMargin = data.get('printheadermargin', _defaultHeaderPos)
+        self.footerMargin = data.get('printfootermargin', _defaultHeaderPos)
+        self.numColumns = data.get('printnumcolumns', 1)
+        self.columnSpacing = data.get('printcolumnspace', _defaultColumnSpace)
+        self.headerText = data.get('printheadertext', '')
+        self.footerText = data.get('printfootertext', '')
+        if 'printfont' in data:
             self.useDefaultFont = False
-            self.mainFont.fromString(attrs['printfont'])
+            self.mainFont.fromString(data['printfont'])
         self.adjustSpacing()
 
     def roundedMargins(self):
-        """Return a list of rounded page margins in inches.
+        """Return a tuple of rounded page margins in inches.
 
         Rounds to nearest .01" to avoid Qt unit conversion artifacts.
         """
-        return [round(margin, 2) for margin in
-                self.printer.getPageMargins(QtGui.QPrinter.Inch)]
+        margins = self.pageLayout.margins(QPageLayout.Inch)
+        return tuple(round(margin, 2) for margin in
+                     (margins.left(), margins.top(), margins.right(),
+                      margins.bottom()))
 
     def roundedPaperSize(self):
         """Return a tuple of rounded paper width and height.
 
         Rounds to nearest .01" to avoid Qt unit conversion artifacts.
         """
-        size = self.printer.paperSize(QtGui.QPrinter.Inch)
+        size = self.pageLayout.fullRect(QPageLayout.Inch)
         return (round(size.width(), 2), round(size.height(), 2))
 
+    def paperSizeName(self, sizeId=None):
+        """Return a QPageSize attribute name matching the paper size ID.
+
+        Arguments:
+            sizeId -- the Qt size ID, if None, use current size
+        """
+        if sizeId == None:
+            sizeId = self.pageLayout.pageSize().id()
+        matches = []
+        for name, num in vars(QPageSize).items():
+            if num == sizeId:
+                matches.append(name)
+        if not matches:
+            return 'Custom'
+        if len(matches) > 1:
+            text = QPageSize(sizeId).name().split(None, 1)[0]
+            for name in matches:
+                if name == text:
+                    return name
+        return matches[0]
+
     def setupData(self):
         """Load data to be printed and set page info.
         """
-        if self.printWhat == entireTree:
-            selNodes = [self.localControl.model.root]
+        if self.printWhat == PrintScope.entireTree:
+            selSpots = self.localControl.structure.rootSpots()
         else:
-            selNodes = (self.localControl.currentSelectionModel().
-                        selectedNodes())
-        self.outputGroup = treeoutput.OutputGroup(selNodes, self.includeRoot,
-                                                  self.printWhat != selectNode,
+            selSpots = (self.localControl.currentSelectionModel().
+                        selectedSpots())
+            if not selSpots:
+                selSpots = self.localControl.structure.rootSpots()
+        self.outputGroup = treeoutput.OutputGroup(selSpots, self.includeRoot,
+                                                  self.printWhat !=
+                                                  PrintScope.selectNode,
                                                   self.openOnly)
         self.paginate()
 
@@ -194,9 +220,12 @@ class PrintData:
         pageNum = 1
         columnNum = 0
         pagePos = 0
-        heightAvail = self.printer.pageRect().height()
+        self.checkPageLayout()
+        heightAvail = (self.pageLayout.paintRect().height() *
+                       self.printer.logicalDpiY())
         columnSpacing = int(self.columnSpacing * self.printer.logicalDpiX())
-        widthAvail = ((self.printer.pageRect().width() -
+        widthAvail = ((self.pageLayout.paintRect().width() *
+                       self.printer.logicalDpiX() -
                        columnSpacing * (self.numColumns - 1)) //
                       self.numColumns)
         newGroup = treeoutput.OutputGroup([])
@@ -234,7 +263,7 @@ class PrintData:
                                                         self.mainFont)
                     if newItem:
                         self.outputGroup.insert(0, newItem)
-            if item and pagePos + item.height <= heightAvail:
+            if item and (pagePos + item.height <= heightAvail or pagePos == 0):
                 item.pageNum = pageNum
                 item.columnNum = columnNum
                 item.pagePos = pagePos
@@ -270,23 +299,72 @@ class PrintData:
         self.outputGroup.loadFamilyRefs()
         self.printer.setFromTo(1, pageNum)
 
+    def checkPageLayout(self):
+        """Check and set the page layout on the current printer.
+
+        Verify that the layout settings match the printer, adjust if required.
+        """
+        if not self.printer.setPageLayout(self.pageLayout):
+            tempPrinter = QPrinter()
+            tempPageLayout = tempPrinter.pageLayout()
+            tempPageLayout.setUnits(QPageLayout.Inch)
+            pageSizeIssue = False
+            defaultPageSize = tempPageLayout.pageSize()
+            tempPageLayout.setPageSize(self.pageLayout.pageSize())
+            if not tempPrinter.setPageLayout(tempPageLayout):
+                pageSizeIssue = True
+                tempPageLayout.setPageSize(defaultPageSize)
+            marginIssue = not (tempPageLayout.setMargins(self.pageLayout.
+                                                         margins()) and
+                               tempPrinter.setPageLayout(tempPageLayout))
+            if marginIssue:
+                margin = 0.1
+                while True:
+                    if (tempPageLayout.setMargins(QMarginsF(*(margin,) * 4))
+                        and tempPrinter.setPageLayout(tempPageLayout)):
+                        break
+                    margin += 0.1
+                newMargins = []
+                for oldMargin in self.roundedMargins():
+                    newMargins.append(oldMargin if oldMargin >= margin else
+                                      margin)
+                tempPageLayout.setMargins(QMarginsF(*newMargins))
+            tempPageLayout.setOrientation(self.pageLayout.orientation())
+            self.printer.setPageLayout(tempPageLayout)
+            if not pageSizeIssue and not marginIssue:
+                return
+            if pageSizeIssue and marginIssue:
+                msg = _('Warning: Page size and margin settings unsupported '
+                        'on current printer.\nSave page adjustments?')
+            elif pageSizeIssue:
+                msg = _('Warning: Page size setting unsupported '
+                        'on current printer.\nSave adjustment?')
+            else:
+                msg = _('Warning: Margin settings unsupported '
+                        'on current printer.\nSave adjustments?')
+            ans = QMessageBox.warning(QApplication.activeWindow(), 'TreeLine',
+                                      msg, QMessageBox.Yes | QMessageBox.No,
+                                      QMessageBox.Yes)
+            if ans == QMessageBox.Yes:
+                self.pageLayout = tempPageLayout
+
     def paintData(self, printer):
         """Paint data to be printed to the printer.
         """
         pageNum = 1
         maxPageNum = self.outputGroup[-1].pageNum
-        if self.printer.printRange() != QtGui.QPrinter.AllPages:
+        if self.printer.printRange() != QPrinter.AllPages:
             pageNum = self.printer.fromPage()
             maxPageNum = self.printer.toPage()
-        painter = QtGui.QPainter()
+        painter = QPainter()
         if not painter.begin(self.printer):
-            QtGui.QMessageBox.warning(QtGui.QApplication.activeWindow(),
-                                   'TreeLine', _('Error initializing printer'))
-        QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
+            QMessageBox.warning(QApplication.activeWindow(),
+                                'TreeLine', _('Error initializing printer'))
+        QApplication.setOverrideCursor(Qt.WaitCursor)
         while True:
             self.paintPage(pageNum, painter)
             if pageNum == maxPageNum:
-                QtGui.QApplication.restoreOverrideCursor()
+                QApplication.restoreOverrideCursor()
                 return
             pageNum += 1
             self.printer.newPage()
@@ -303,18 +381,20 @@ class PrintData:
         if headerDoc:
             layout = headerDoc.documentLayout()
             layout.setPaintDevice(self.printer)
-            headerDoc.setTextWidth(self.printer.pageRect().width())
+            headerDoc.setTextWidth(self.pageLayout.paintRect().width() *
+                                   self.printer.logicalDpiX())
             painter.save()
-            topMargin = self.printer.getPageMargins(QtGui.QPrinter.Inch)[1]
+            topMargin = self.pageLayout.margins(QPageLayout.Inch).top()
             headerDelta = ((self.headerMargin - topMargin) *
                            self.printer.logicalDpiX())
             painter.translate(0, int(headerDelta))
             layout.draw(painter,
-                        QtGui.QAbstractTextDocumentLayout.PaintContext())
+                        QAbstractTextDocumentLayout.PaintContext())
             painter.restore()
         painter.save()
         columnSpacing = int(self.columnSpacing * self.printer.logicalDpiX())
-        columnDelta = ((self.printer.pageRect().width() -
+        columnDelta = ((self.pageLayout.paintRect().width() *
+                        self.printer.logicalDpiX() -
                         columnSpacing * (self.numColumns - 1)) /
                        self.numColumns) + columnSpacing
         for columnNum in range(self.numColumns):
@@ -326,15 +406,17 @@ class PrintData:
         if footerDoc:
             layout = footerDoc.documentLayout()
             layout.setPaintDevice(self.printer)
-            footerDoc.setTextWidth(self.printer.pageRect().width())
+            footerDoc.setTextWidth(self.pageLayout.paintRect().width() *
+                                   self.printer.logicalDpiX())
             painter.save()
-            bottomMargin = self.printer.getPageMargins(QtGui.QPrinter.Inch)[3]
+            bottomMargin = self.pageLayout.margins(QPageLayout.Inch).bottom()
             footerDelta = ((bottomMargin - self.footerMargin) *
                            self.printer.logicalDpiX())
-            painter.translate(0, self.printer.pageRect().height() +
-                                 int(footerDelta) - self.lineSpacing)
+            painter.translate(0, self.pageLayout.paintRect().height() *
+                              self.printer.logicalDpiX() +
+                              int(footerDelta) - self.lineSpacing)
             layout.draw(painter,
-                        QtGui.QAbstractTextDocumentLayout.PaintContext())
+                        QAbstractTextDocumentLayout.PaintContext())
             painter.restore()
 
     def paintColumn(self, pageNum, columnNum, painter):
@@ -352,7 +434,7 @@ class PrintData:
             painter.save()
             painter.translate(item.level * self.indentSize, item.pagePos)
             layout.draw(painter,
-                        QtGui.QAbstractTextDocumentLayout.PaintContext())
+                        QAbstractTextDocumentLayout.PaintContext())
             painter.restore()
         if self.drawLines:
             self.addPrintLines(pageNum, columnNum, columnItems, painter)
@@ -369,7 +451,8 @@ class PrintData:
         parentsDrawn = set()
         horizOffset = self.indentSize // 2
         vertOffset = self.lineSpacing // 2
-        heightAvail = self.printer.pageRect().height()
+        heightAvail = (self.pageLayout.paintRect().height() *
+                       self.printer.logicalDpiY())
         for item in columnItems:
             if item.level > 0:
                 indent = item.level * self.indentSize
@@ -411,9 +494,10 @@ class PrintData:
             textParts = printdialogs.splitHeaderFooter(self.footerText)
         if not textParts:
             return ''
-        fileInfoFormat = self.localControl.model.formats.fileInfoFormat
-        fileInfoNode = self.localControl.model.fileInfoNode
-        fileInfoFormat.updateFileInfo(self.localControl.filePath, fileInfoNode)
+        fileInfoFormat = self.localControl.structure.treeFormats.fileInfoFormat
+        fileInfoNode = self.localControl.structure.fileInfoNode
+        fileInfoFormat.updateFileInfo(self.localControl.filePathObj,
+                                      fileInfoNode)
         fileInfoNode.data[fileInfoFormat.pageNumFieldName] = repr(pageNum)
         fileInfoNode.data[fileInfoFormat.numPagesFieldName] = repr(numPages)
         fileInfoFormat.changeOutputLines(textParts, keepBlanks=True)
@@ -437,7 +521,7 @@ class PrintData:
         """
         text = self.formatHeaderFooter(header, pageNum, numPages)
         if text:
-            doc = QtGui.QTextDocument()
+            doc = QTextDocument()
             doc.setHtml(text)
             doc.setDefaultFont(self.mainFont)
             frameFormat = doc.rootFrame().frameFormat()
@@ -451,8 +535,7 @@ class PrintData:
     def printSetup(self):
         """Show a dialog to set margins, page size and other printing options.
         """
-        setupDialog = printdialogs.PrintSetupDialog(self, True,
-                                                    QtGui.QApplication.
+        setupDialog = printdialogs.PrintSetupDialog(self, True, QApplication.
                                                     activeWindow())
         setupDialog.exec_()
 
@@ -460,21 +543,20 @@ class PrintData:
         """Show a preview of printing results.
         """
         self.setupData()
-        previewDialog = printdialogs.PrintPreviewDialog(self,
-                                                        QtGui.QApplication.
+        previewDialog = printdialogs.PrintPreviewDialog(self,QApplication.
                                                         activeWindow())
         previewDialog.previewWidget.paintRequested.connect(self.paintData)
-        if globalref.genOptions.getValue('SaveWindowGeom'):
+        if globalref.genOptions['SaveWindowGeom']:
             previewDialog.restoreDialogGeom()
         previewDialog.exec_()
 
     def filePrint(self):
         """Show dialog and print tree output based on current options.
         """
+        self.printer.setOutputFormat(QPrinter.NativeFormat)
         self.setupData()
-        printDialog = QtGui.QPrintDialog(self.printer,
-                                         QtGui.QApplication.activeWindow())
-        if printDialog.exec_() == QtGui.QDialog.Accepted:
+        printDialog = QPrintDialog(self.printer, QApplication.activeWindow())
+        if printDialog.exec_() == QDialog.Accepted:
             self.paintData(self.printer)
 
     def filePrintPdf(self):
@@ -482,11 +564,11 @@ class PrintData:
         """
         filters = ';;'.join((globalref.fileFilters['pdf'],
                              globalref.fileFilters['all']))
-        defaultFilePath = globalref.mainControl.defaultFilePath()
+        defaultFilePath = str(globalref.mainControl.defaultPathObj())
         defaultFilePath = os.path.splitext(defaultFilePath)[0]
         if os.path.basename(defaultFilePath):
             defaultFilePath = '{0}.{1}'.format(defaultFilePath, 'pdf')
-        filePath = QtGui.QFileDialog.getSaveFileName(QtGui.QApplication.
+        filePath, selectFilter = QFileDialog.getSaveFileName(QApplication.
                                                     activeWindow(),
                                                     _('TreeLine - Export PDF'),
                                                     defaultFilePath, filters)
@@ -494,11 +576,12 @@ class PrintData:
             return
         if not os.path.splitext(filePath)[1]:
             filePath = '{0}.{1}'.format(filePath, 'pdf')
-        self.printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
+        origFormat = self.printer.outputFormat()
+        self.printer.setOutputFormat(QPrinter.PdfFormat)
         self.printer.setOutputFileName(filePath)
         self.adjustSpacing()
         self.setupData()
         self.paintData(self.printer)
-        self.printer.setOutputFormat(QtGui.QPrinter.NativeFormat)
+        self.printer.setOutputFormat(origFormat)
         self.printer.setOutputFileName('')
         self.adjustSpacing()
