@@ -1,22 +1,27 @@
 #!/usr/bin/env python3
 
-#****************************************************************************
+#******************************************************************************
 # spellcheck.py, provides classes for spell check interfaces and dialogs,
 # including interfaces to aspell, ispell, hunspell.
 #
-# Copyright (C) 2014, Douglas W. Bell
+# TreeLine, an information storage program
+# Copyright (C) 2017, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
 # but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
-#*****************************************************************************
+#******************************************************************************
 
 import re
 import sys
 import subprocess
 import collections
-from PyQt4 import QtCore, QtGui
+from PyQt5.QtCore import QSize, Qt, pyqtSignal
+from PyQt5.QtGui import QFontMetrics, QTextCursor
+from PyQt5.QtWidgets import (QApplication, QDialog, QFileDialog, QGroupBox,
+                             QHBoxLayout, QLabel, QLineEdit, QListWidget,
+                             QMessageBox, QPushButton, QTextEdit, QVBoxLayout)
 import undo
 import globalref
 
@@ -175,12 +180,12 @@ class SpellCheckOperation:
         """
         self.controlRef = controlRef
         self.selectModel = controlRef.currentSelectionModel()
-        self.currentNode = None
+        self.currentSpot = None
         self.currentField = ''
         self.lineNum = 0
         self.textLine = ''
-        parentWidget = QtGui.QApplication.activeWindow()
-        path = globalref.miscOptions.getValue('SpellCheckPath')
+        parentWidget = QApplication.activeWindow()
+        path = globalref.miscOptions['SpellCheckPath']
         while True:
             try:
                 self.spellCheckInterface = SpellCheckInterface(path,
@@ -195,17 +200,17 @@ class SpellCheckOperation:
                         prompt = (_('Could not find either aspell.exe, '
                                     'ispell.exe or hunspell.exe\n'
                                     'Browse for location?'))
-                        ans = QtGui.QMessageBox.warning(parentWidget,
+                        ans = QMessageBox.warning(parentWidget,
                                                       _('Spell Check Error'),
                                                       prompt,
-                                                      QtGui.QMessageBox.Yes |
-                                                      QtGui.QMessageBox.Cancel,
-                                                      QtGui.QMessageBox.Yes)
-                        if ans == QtGui.QMessageBox.Cancel:
+                                                      QMessageBox.Yes |
+                                                      QMessageBox.Cancel,
+                                                      QMessageBox.Yes)
+                        if ans == QMessageBox.Cancel:
                             raise
                         title = _('Locate aspell.exe, ipsell.exe or '
                                   'hunspell.exe')
-                        path = QtGui.QFileDialog.getOpenFileName(parentWidget,
+                        path, fltr = QFileDialog.getOpenFileName(parentWidget,
                                                           title, '',
                                                           _('Program (*.exe)'))
                         if path:
@@ -218,47 +223,44 @@ class SpellCheckOperation:
                     else:
                         prompt = (_('TreeLine Spell Check Error\nMake sure '
                                     'aspell, ispell or hunspell is installed'))
-                        QtGui.QMessageBox.warning(parentWidget, 'TreeLine',
-                                                  prompt)
+                        QMessageBox.warning(parentWidget, 'TreeLine', prompt)
                         raise
 
     def spellCheck(self):
         """Spell check starting with the selected branches.
         """
-        parentWidget = QtGui.QApplication.activeWindow()
+        parentWidget = QApplication.activeWindow()
         spellCheckDialog = SpellCheckDialog(self.spellCheckInterface,
                                             parentWidget)
         spellCheckDialog.misspellFound.connect(self.updateSelection)
         spellCheckDialog.changeRequest.connect(self.changeNode)
-        origBranches = self.selectModel.uniqueBranches()
+        origBranches = self.selectModel.selectedBranchSpots()
+        if not origBranches:
+            origBranches = self.controlRef.structure.rootSpots()
         result = (spellCheckDialog.
                   startSpellCheck(self.textLineGenerator(origBranches)))
-        self.selectModel.selectNodes(origBranches, expandParents = True)
-        if result and origBranches[0].parent:
-            prompt = (_('Finished checking the branch\n'
-                        'Continue from the root branch?'))
-            ans = QtGui.QMessageBox.information(parentWidget,
-                                                _('TreeLine Spell Check'),
-                                                prompt,
-                                                QtGui.QMessageBox.Yes |
-                                                QtGui.QMessageBox.No)
-            if ans == QtGui.QMessageBox.Yes:
-                generator = self.textLineGenerator([self.controlRef.model.
-                                                    root])
+        self.selectModel.selectSpots(origBranches, expandParents = True)
+        if result and origBranches[0].parentSpot.parentSpot:
+            prompt = _('Finished checking the branch\nContinue from the top?')
+            ans = QMessageBox.information(parentWidget,
+                                          _('TreeLine Spell Check'), prompt,
+                                          QMessageBox.Yes | QMessageBox.No)
+            if ans == QMessageBox.Yes:
+                generator = self.textLineGenerator(self.controlRef.structure.
+                                                   rootSpots())
                 result = spellCheckDialog.startSpellCheck(generator)
-                self.selectModel.selectNodes(origBranches,
+                self.selectModel.selectSpots(origBranches,
                                              expandParents = True)
             else:
                 result = False
         if result:
-            QtGui.QMessageBox.information(parentWidget,
-                                          _('TreeLine Spell Check'),
-                                          _('Finished spell checking'))
+            QMessageBox.information(parentWidget, _('TreeLine Spell Check'),
+                                    _('Finished spell checking'))
 
     def updateSelection(self):
         """Change the tree selection to the node with a misspelled word.
         """
-        self.selectModel.selectNode(self.currentNode, expandParents = True)
+        self.selectModel.selectSpots([self.currentSpot], expandParents = True)
 
     def changeNode(self, newTextLine):
         """Replace the current text line in the current node.
@@ -266,12 +268,12 @@ class SpellCheckOperation:
         Arguments:
             newTextLine -- the new text to use
         """
-        undo.DataUndo(self.currentNode.modelRef.undoList, self.currentNode)
-        textLines = (self.currentNode.data.get(self.currentField, '').
-                     split('\n'))
+        node = self.currentSpot.nodeRef
+        undo.DataUndo(self.controlRef.structure.undoList, node)
+        textLines = node.data.get(self.currentField, '').split('\n')
         textLines[self.lineNum] = newTextLine
-        self.currentNode.data[self.currentField] = '\n'.join(textLines)
-        self.controlRef.updateTreeNode(self.currentNode)
+        node.data[self.currentField] = '\n'.join(textLines)
+        self.controlRef.updateTreeNode(node)
 
     def textLineGenerator(self, branches):
         """Yield next line to be checked.
@@ -280,21 +282,21 @@ class SpellCheckOperation:
             branches -- a list of branch parent nodes to check.
         """
         for parent in branches:
-            for self.currentNode in parent.descendantGen():
-                for self.currentField in (self.currentNode.nodeFormat().
-                                          fieldNames()):
-                    text = self.currentNode.data.get(self.currentField, '')
+            for self.currentSpot in parent.spotDescendantGen():
+                node = self.currentSpot.nodeRef
+                for self.currentField in node.formatRef.fieldNames():
+                    text = node.data.get(self.currentField, '')
                     if text:
                         for self.lineNum, self.textLine in \
                                             enumerate(text.split('\n')):
                             yield self.textLine
 
 
-class SpellCheckDialog(QtGui.QDialog):
+class SpellCheckDialog(QDialog):
     """Dialog to perform and control the spell check operation.
     """
-    misspellFound = QtCore.pyqtSignal()
-    changeRequest = QtCore.pyqtSignal(str)
+    misspellFound = pyqtSignal()
+    changeRequest = pyqtSignal(str)
     def __init__(self, spellCheckInterface, parent=None):
         """Create the dialog.
 
@@ -303,8 +305,8 @@ class SpellCheckDialog(QtGui.QDialog):
             parent -- the parent dialog
         """
         super().__init__(parent)
-        self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint |
-                            QtCore.Qt.WindowSystemMenuHint)
+        self.setWindowFlags(Qt.Dialog | Qt.WindowTitleHint |
+                            Qt.WindowCloseButtonHint)
         self.setWindowTitle(_('Spell Check'))
         self.spellCheckInterface = spellCheckInterface
         self.textLineIter = None
@@ -314,55 +316,55 @@ class SpellCheckDialog(QtGui.QDialog):
         self.word = ''
         self.postion = 0
 
-        topLayout = QtGui.QHBoxLayout(self)
-        leftLayout = QtGui.QVBoxLayout()
+        topLayout = QHBoxLayout(self)
+        leftLayout = QVBoxLayout()
         topLayout.addLayout(leftLayout)
-        wordBox = QtGui.QGroupBox(_('Not in Dictionary'))
+        wordBox = QGroupBox(_('Not in Dictionary'))
         leftLayout.addWidget(wordBox)
-        wordLayout = QtGui.QVBoxLayout(wordBox)
-        label = QtGui.QLabel(_('Word:'))
+        wordLayout = QVBoxLayout(wordBox)
+        label = QLabel(_('Word:'))
         wordLayout.addWidget(label)
-        self.wordEdit = QtGui.QLineEdit()
+        self.wordEdit = QLineEdit()
         wordLayout.addWidget(self.wordEdit)
         self.wordEdit.textChanged.connect(self.updateFromWord)
         wordLayout.addSpacing(5)
-        label = QtGui.QLabel(_('Context:'))
+        label = QLabel(_('Context:'))
         wordLayout.addWidget(label)
         self.contextEdit = SpellContextEdit()
         wordLayout.addWidget(self.contextEdit)
         self.contextEdit.textChanged.connect(self.updateFromContext)
 
-        suggestBox = QtGui.QGroupBox(_('Suggestions'))
+        suggestBox = QGroupBox(_('Suggestions'))
         leftLayout.addWidget(suggestBox)
-        suggestLayout =  QtGui.QVBoxLayout(suggestBox)
-        self.suggestList = QtGui.QListWidget()
+        suggestLayout =  QVBoxLayout(suggestBox)
+        self.suggestList = QListWidget()
         suggestLayout.addWidget(self.suggestList)
         self.suggestList.itemDoubleClicked.connect(self.replace)
 
-        rightLayout = QtGui.QVBoxLayout()
+        rightLayout = QVBoxLayout()
         topLayout.addLayout(rightLayout)
-        ignoreButton = QtGui.QPushButton(_('Ignor&e'))
+        ignoreButton = QPushButton(_('Ignor&e'))
         rightLayout.addWidget(ignoreButton)
         ignoreButton.clicked.connect(self.ignore)
-        ignoreAllButton = QtGui.QPushButton(_('&Ignore All'))
+        ignoreAllButton = QPushButton(_('&Ignore All'))
         rightLayout.addWidget(ignoreAllButton)
         ignoreAllButton.clicked.connect(self.ignoreAll)
         rightLayout.addStretch()
-        addButton = QtGui.QPushButton(_('&Add'))
+        addButton = QPushButton(_('&Add'))
         rightLayout.addWidget(addButton)
         addButton.clicked.connect(self.add)
-        addLowerButton = QtGui.QPushButton(_('Add &Lowercase'))
+        addLowerButton = QPushButton(_('Add &Lowercase'))
         rightLayout.addWidget(addLowerButton)
         addLowerButton.clicked.connect(self.addLower)
         rightLayout.addStretch()
-        replaceButton = QtGui.QPushButton(_('&Replace'))
+        replaceButton = QPushButton(_('&Replace'))
         rightLayout.addWidget(replaceButton)
         replaceButton.clicked.connect(self.replace)
-        self.replaceAllButton = QtGui.QPushButton(_('Re&place All'))
+        self.replaceAllButton = QPushButton(_('Re&place All'))
         rightLayout.addWidget(self.replaceAllButton)
         self.replaceAllButton.clicked.connect(self.replaceAll)
         rightLayout.addStretch()
-        cancelButton = QtGui.QPushButton(_('&Cancel'))
+        cancelButton = QPushButton(_('&Cancel'))
         rightLayout.addWidget(cancelButton)
         cancelButton.clicked.connect(self.reject)
         self.widgetDisableList = [ignoreButton, ignoreAllButton, addButton,
@@ -384,7 +386,7 @@ class SpellCheckDialog(QtGui.QDialog):
         except StopIteration:
             return True
         if self.spellCheck():
-            if self.exec_() == QtGui.QDialog.Rejected:
+            if self.exec_() == QDialog.Rejected:
                 return False
         return True
 
@@ -525,7 +527,7 @@ class SpellCheckDialog(QtGui.QDialog):
         self.suggestList.clearSelection()
 
 
-class SpellContextEdit(QtGui.QTextEdit):
+class SpellContextEdit(QTextEdit):
     """Editor for spell check word context.
     
     Sets the size hint to 3 lines and simplifies selction.
@@ -542,8 +544,8 @@ class SpellContextEdit(QtGui.QTextEdit):
     def sizeHint(self):
         """Set prefered size of 3 lines long.
         """
-        fontHeight = QtGui.QFontMetrics(self.currentFont()).lineSpacing()
-        return QtCore.QSize(QtGui.QTextEdit.sizeHint(self).width(),
+        fontHeight = QFontMetrics(self.currentFont()).lineSpacing()
+        return QSize(QTextEdit.sizeHint(self).width(),
                             fontHeight * 3)
 
     def setSelection(self, fromPos, toPos):
@@ -555,6 +557,6 @@ class SpellContextEdit(QtGui.QTextEdit):
         """
         cursor = self.textCursor()
         cursor.setPosition(fromPos)
-        cursor.setPosition(toPos, QtGui.QTextCursor.KeepAnchor)
+        cursor.setPosition(toPos, QTextCursor.KeepAnchor)
         self.setTextCursor(cursor)
         self.ensureCursorVisible()
